@@ -8,13 +8,13 @@
 #include "x11lgnd.h"
 #include "x11axis.h"
 #include "x11file.h"
+#include "x11parameter.h"
 #include "x11graph.h"
 #include "x11print.h"
 #include "x11opt.h"
 #include "x11view.h"
 #include "ox11menu.h"
 
-#if USE_APP_MENU
 static void
 help_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
 {
@@ -27,6 +27,12 @@ about_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
   CmHelpAbout(NULL, NULL);
 }
 
+static void
+demo_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
+{
+  CmHelpDemo(NULL, NULL);
+}
+
 static void
 quit_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
 {
@@ -63,12 +69,6 @@ GraphLoadAction_activated(GSimpleAction *action, GVariant *parameter, gpointer a
   CmGraphLoad(NULL, NULL);
 }
 
-static void
-GraphRecentAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
-{
-  show_recent_dialog(RECENT_TYPE_GRAPH);
-}
-
 static void
 GraphSaveAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
 {
@@ -117,6 +117,20 @@ GraphExportPNGAction_activated(GSimpleAction *action, GVariant *parameter, gpoin
   CmOutputMenu(NULL, GINT_TO_POINTER(MenuIdOutputPNGFile));
 }
 
+#if WINDOWS
+static void
+GraphExportEMFAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
+{
+  CmOutputMenu(NULL, GINT_TO_POINTER(MenuIdOutputEMFFile));
+}
+
+static void
+GraphExportEMFClipboardAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
+{
+  CmOutputMenu(NULL, GINT_TO_POINTER(MenuIdOutputEMFClipboard));
+}
+#endif
+
 static void
 GraphDrawOrderAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
 {
@@ -196,7 +210,7 @@ EditCutAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app
 static void
 EditCopyAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
 {
-  CmEditMenuCB(NULL, GINT_TO_POINTER(MenuIdEditCut));
+  CmEditMenuCB(NULL, GINT_TO_POINTER(MenuIdEditCopy));
 }
 
 static void
@@ -217,6 +231,12 @@ EditDuplicateAction_activated(GSimpleAction *action, GVariant *parameter, gpoint
   CmEditMenuCB(NULL, GINT_TO_POINTER(MenuIdEditDuplicate));
 }
 
+static void
+EditSelectAllAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
+{
+  CmEditMenuCB(NULL, GINT_TO_POINTER(MenuIdEditSelectAll));
+}
+
 static void
 EditOrderTopAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
 {
@@ -295,7 +315,7 @@ EditFlipHAction_activated(GSimpleAction *action, GVariant *parameter, gpointer a
 }
 
 static void
-EditFlipVActiopn_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
+EditFlipVAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
 {
   CmEditMenuCB(NULL, GINT_TO_POINTER(MenuIdEditFlipVertically));
 }
@@ -418,12 +438,6 @@ DataAddRangeAction_activated(GSimpleAction *action, GVariant *parameter, gpointe
   CmRangeAdd(NULL, NULL);
 }
 
-static void
-DataAddRecentFileAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
-{
-  show_recent_dialog(RECENT_TYPE_DATA);
-}
-
 static void
 DataPropertyAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
 {
@@ -454,28 +468,46 @@ DataMathAction_activated(GSimpleAction *action, GVariant *parameter, gpointer ap
   CmFileMath(NULL, NULL);
 }
 
+static void
+ParameterAddAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
+{
+  CmParameterAdd(NULL, NULL);
+}
+
+static void
+ParameterPropertyAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
+{
+  CmParameterUpdate(NULL, NULL);
+}
+
+static void
+ParameterDeleteAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
+{
+  CmParameterDelete(NULL, NULL);
+}
+
 static void
 AxisAddFrameAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
 {
-  CmAxisNewFrame(NULL, NULL);
+  CmAxisAddFrame(NULL, NULL);
 }
 
 static void
 AxisAddSectionAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
 {
-  CmAxisNewSection(NULL, NULL);
+  CmAxisAddSection(NULL, NULL);
 }
 
 static void
 AxisAddCrossAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
 {
-  CmAxisNewCross(NULL, NULL);
+  CmAxisAddCross(NULL, NULL);
 }
 
 static void
 AxisAddSingleAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
 {
-  CmAxisNewSingle(NULL, NULL);
+  CmAxisAddSingle(NULL, NULL);
 }
 
 static void
@@ -668,13 +700,13 @@ static GActionEntry AppEntries[] =
 {
   { "help", help_activated, NULL, NULL, NULL },
   { "about", about_activated, NULL, NULL, NULL },
+  { "demo", demo_activated, NULL, NULL, NULL },
   { "quit", quit_activated, NULL, NULL, NULL },
   { "preferences", PreferenceMiscAction_activated, NULL, NULL, NULL },
   { "GraphNewFrameAction", GraphNewFrameAction_activated, NULL, NULL, NULL },
   { "GraphNewSectionAction", GraphNewSectionAction_activated, NULL, NULL, NULL },
   { "GraphNewCrossAction", GraphNewCrossAction_activated, NULL, NULL, NULL },
   { "GraphNewClearAction", GraphNewClearAction_activated, NULL, NULL, NULL },
-  { "GraphRecentAction", GraphRecentAction_activated, NULL, NULL, NULL },
   { "GraphLoadAction", GraphLoadAction_activated, NULL, NULL, NULL },
   { "GraphSaveAction", GraphSaveAction_activated, NULL, NULL, NULL },
   { "GraphSaveAsAction", GraphSaveAsAction_activated, NULL, NULL, NULL },
@@ -684,6 +716,10 @@ static GActionEntry AppEntries[] =
   { "GraphExportPDFAction", GraphExportPDFAction_activated, NULL, NULL, NULL },
   { "GraphExportSVGAction", GraphExportSVGAction_activated, NULL, NULL, NULL },
   { "GraphExportPNGAction", GraphExportPNGAction_activated, NULL, NULL, NULL },
+#if WINDOWS
+  { "GraphExportEMFAction", GraphExportEMFAction_activated, NULL, NULL, NULL },
+  { "GraphExportEMFClipboardAction", GraphExportEMFClipboardAction_activated, NULL, NULL, NULL },
+#endif
   { "GraphDrawOrderAction", GraphDrawOrderAction_activated, NULL, NULL, NULL },
   { "GraphPageSetupAction", GraphPageSetupAction_activated, NULL, NULL, NULL },
   { "GraphPrintPreviewAction", GraphPrintPreviewAction_activated, NULL, NULL, NULL },
@@ -698,6 +734,7 @@ static GActionEntry AppEntries[] =
   { "EditPasteAction", EditPasteAction_activated, NULL, NULL, NULL },
   { "EditDeleteAction", EditDeleteAction_activated, NULL, NULL, NULL },
   { "EditDuplicateAction", EditDuplicateAction_activated, NULL, NULL, NULL },
+  { "EditSelectAllAction", EditSelectAllAction_activated, NULL, NULL, NULL },
   { "EditOrderTopAction", EditOrderTopAction_activated, NULL, NULL, NULL },
   { "EditOrderUpAction", EditOrderUpAction_activated, NULL, NULL, NULL },
   { "EditOrderDownAction", EditOrderDownAction_activated, NULL, NULL, NULL },
@@ -711,7 +748,7 @@ static GActionEntry AppEntries[] =
   { "EditRotateCWAction", EditRotateCWAction_activated, NULL, NULL, NULL },
   { "EditRotateCCWAction", EditRotateCCWAction_activated, NULL, NULL, NULL },
   { "EditFlipHAction", EditFlipHAction_activated, NULL, NULL, NULL },
-  { "EditFlipVActiopn", EditFlipVActiopn_activated, NULL, NULL, NULL },
+  { "EditFlipVAction", EditFlipVAction_activated, NULL, NULL, NULL },
   { "ViewDrawDirectAction", ViewDrawDirectAction_activated, NULL, NULL, NULL },
   { "ViewDrawAction", ViewDrawAction_activated, NULL, NULL, NULL },
   { "ViewClearInformationWindowAction", ViewClearInformationWindowAction_activated, NULL, NULL, NULL },
@@ -725,12 +762,14 @@ static GActionEntry AppEntries[] =
   { "ViewGridLineAction", NULL, NULL, "true", ViewGridLineAction_activated },
   { "DataAddFileAction", DataAddFileAction_activated, NULL, NULL, NULL },
   { "DataAddRangeAction", DataAddRangeAction_activated, NULL, NULL, NULL },
-  { "DataAddRecentFileAction", DataAddRecentFileAction_activated, NULL, NULL, NULL },
   { "DataPropertyAction", DataPropertyAction_activated, NULL, NULL, NULL },
   { "DataCloseAction", DataCloseAction_activated, NULL, NULL, NULL },
   { "DataEditAction", DataEditAction_activated, NULL, NULL, NULL },
   { "DataSaveAction", DataSaveAction_activated, NULL, NULL, NULL },
   { "DataMathAction", DataMathAction_activated, NULL, NULL, NULL },
+  { "ParameterAddAction", ParameterAddAction_activated, NULL, NULL, NULL },
+  { "ParameterPropertyAction", ParameterPropertyAction_activated, NULL, NULL, NULL },
+  { "ParameterDeleteAction", ParameterDeleteAction_activated, NULL, NULL, NULL },
   { "AxisAddFrameAction", AxisAddFrameAction_activated, NULL, NULL, NULL },
   { "AxisAddSectionAction", AxisAddSectionAction_activated, NULL, NULL, NULL },
   { "AxisAddCrossAction", AxisAddCrossAction_activated, NULL, NULL, NULL },
@@ -768,39 +807,14 @@ static GActionEntry AppEntries[] =
   { "PopupUpdateAction", PopupUpdateAction_activated, NULL, NULL, NULL },
 };
 
-GtkApplication *
-create_application_window(GtkWidget **popup)
+static int Initialized = FALSE;
+
+void
+setup_actions(GtkApplication *app)
 {
-  GtkApplication *app;
-
-  app = gtk_application_new(APPLICATION_ID, G_APPLICATION_NON_UNIQUE);
-  g_application_register(G_APPLICATION(app), NULL, NULL);
+  if (Initialized) {
+    return;
+  }
   g_action_map_add_action_entries(G_ACTION_MAP(app), AppEntries, G_N_ELEMENTS(AppEntries), app);
-
-#if OSX
-  {
-    /* only for remove "Settings" menu item */
-    GtkBuilder *builder;
-    GObject *menu;
-    builder = gtk_builder_new_from_resource(RESOURCE_PATH "/gtk/menus-appmenu.ui");
-    menu = gtk_builder_get_object(builder, "app-menu");
-    gtk_application_set_app_menu(app, G_MENU_MODEL(menu));
-    g_object_unref(builder);
-  }
-#endif
-
-#if USE_GTK_BUILDER
-  /*
-  menu = gtk_application_get_menu_by_id(app, "menubar");
-  gtk_application_set_menubar(app, G_MENU_MODEL(menu));
-  */
-  {
-    GMenu *menu;
-    menu = gtk_application_get_menu_by_id(app, "popup-menu");
-    *popup = gtk_menu_new_from_model(G_MENU_MODEL(menu));
-  }
-#endif
-
-  return app;
+  Initialized = TRUE;
 }
-#endif	/* USE_APP_MENU */
