@@ -14,11 +14,13 @@
 #define GTK_WIDGET_VISIBLE(w) gtk_widget_get_visible(w)
 #endif
 
-#define USE_APP_MENU GTK_CHECK_VERSION(3, 12, 0)
-#if OSX
-#define USE_GTK_BUILDER 1
-#else
-#define USE_GTK_BUILDER 0
+#ifdef GTK_SOURCE_CHECK_VERSION
+#undef GTK_SOURCE_CHECK_VERSION
+#define GTK_SOURCE_CHECK_VERSION(major, minor, micro) \
+	(GTK_SOURCE_MAJOR_VERSION > (major) || \
+	(GTK_SOURCE_MAJOR_VERSION == (major) && GTK_SOURCE_MINOR_VERSION > (minor)) || \
+	(GTK_SOURCE_MAJOR_VERSION == (major) && GTK_SOURCE_MINOR_VERSION == (minor) && \
+	 GTK_SOURCE_MICRO_VERSION >= (micro)))
 #endif
 
 #if WINDOWS || OSX
@@ -27,6 +29,12 @@
 #define USE_HEADER_BAR 1
 #endif
 
+#if GTK_CHECK_VERSION(3, 24, 0)
+#define USE_EVENT_CONTROLLER 1
+#else
+#define USE_EVENT_CONTROLLER 0
+#endif
+
 #define RECENT_CHOOSER_LIMIT 25
 
 #define APPLICATION_ID "com.github.htrb.ngraph-gtk"
@@ -34,7 +42,4 @@
 
 #define CAIRO_COORDINATE_OFFSET 1
 
-#define LINE_NUMBER_WIDGET_NAME "line_number"
-
 #endif
-
