@@ -2,11 +2,12 @@
  * $Id: gtk_liststore.c,v 1.27 2010-02-03 01:18:12 hito Exp $
  */
 
+#include "gtk_common.h"
+
 #include <stdlib.h>
 
 #include "gra.h"
 
-#include "gtk_common.h"
 #include "gtk_liststore.h"
 
 #include "x11dialg.h"
@@ -140,12 +141,12 @@ add_font_combo_item_to_cbox(GtkTreeStore *list, GtkTreeIter *iter, GtkTreeIter *
 {
   struct fontmap *fcur;
   char *font;
-  int match;
 
   getobj(obj, field, id, 0, NULL, &font);
 
   fcur = Gra2cairoConf->fontmap_list_root;
   while (fcur) {
+    int match;
     match = ! g_strcmp0(font, fcur->fontalias);
     add_text_combo_item_to_cbox(list, iter, parent, column_id, -1, fcur->fontalias, TOGGLE_RADIO, match);
     fcur = fcur->next;
@@ -258,7 +259,7 @@ void
 add_line_style_item_to_cbox(GtkTreeStore *list, GtkTreeIter *parent, int column_id, struct objlist *obj, const char *field, int id)
 {
   GtkTreeIter iter;
-  int i, active;
+  int i;
   char *str;
 
   sgetobjfield(obj, id, field, NULL, &str, FALSE, FALSE, FALSE);
@@ -268,6 +269,7 @@ add_line_style_item_to_cbox(GtkTreeStore *list, GtkTreeIter *parent, int column_
 
   add_text_combo_item_to_cbox(list, &iter, parent, -1, -1, _("Line style"), TOGGLE_NONE, FALSE);
   for (i = 0; FwLineStyle[i].name; i++) {
+    int active;
     active = ! g_strcmp0(str, FwLineStyle[i].list);
     add_text_combo_item_to_cbox(list, NULL, &iter, column_id, i, _(FwLineStyle[i].name), TOGGLE_RADIO, active);
   }
@@ -306,11 +308,7 @@ create_column(n_list_store *list, int i)
   case G_TYPE_UINT64:
   case G_TYPE_FLOAT:
   case G_TYPE_DOUBLE:
-#if GTK_CHECK_VERSION(3, 8, 0)
     renderer = gtk_cell_renderer_spin_new();
-#else
-    renderer = gtk_cell_renderer_text_new();
-#endif
     col = gtk_tree_view_column_new_with_attributes(_(list[i].title), renderer,
 						   "text", i, NULL);
     gtk_tree_view_column_set_resizable(col, TRUE);
@@ -319,7 +317,6 @@ create_column(n_list_store *list, int i)
       if (list[i].type == G_TYPE_DOUBLE || list[i].type == G_TYPE_FLOAT) {
 	g_object_set((GObject *) renderer,
 		     "editable", list[i].editable,
-#if GTK_CHECK_VERSION(3, 8, 0)
 		     "adjustment", gtk_adjustment_new(0,
 						      list[i].min / 100.0,
 						      list[i].max / 100.0,
@@ -327,13 +324,11 @@ create_column(n_list_store *list, int i)
 						      list[i].page / 100.0,
 						      0),
 		     "digits", 2,
-#endif
 		     NULL);
 	g_object_set_data(G_OBJECT(renderer), "user-data", &list[i]);
       } else {
 	g_object_set((GObject *) renderer,
 		     "editable", list[i].editable,
-#if GTK_CHECK_VERSION(3, 8, 0)
 		     "adjustment", gtk_adjustment_new(0,
 						      list[i].min,
 						      list[i].max,
@@ -341,7 +336,6 @@ create_column(n_list_store *list, int i)
 						      list[i].page,
 						      0),
 		     "digits", 0,
-#endif
 		     NULL);
 	g_object_set_data(G_OBJECT(renderer), "user-data", &list[i]);
 
@@ -411,10 +405,10 @@ create_column(n_list_store *list, int i)
 void
 tree_view_set_no_expand_column(GtkWidget *tview, const int *columns, int n)
 {
-  GtkTreeViewColumn *col;
   int i;
 
   for (i = 0; i < n; i++) {
+    GtkTreeViewColumn *col;
     col = gtk_tree_view_get_column(GTK_TREE_VIEW(tview), columns[i]);
     gtk_tree_view_column_set_expand(col, FALSE);
   }
@@ -503,7 +497,6 @@ list_store_set_sort_all(GtkWidget *tview)
 {
   GList *list, *ptr;
   int i;
-  GtkTreeViewColumn *column;
 
   list = gtk_tree_view_get_columns(GTK_TREE_VIEW(tview));
 
@@ -511,6 +504,7 @@ list_store_set_sort_all(GtkWidget *tview)
     return;
 
   for (ptr = list, i = 0; ptr; ptr = ptr->next, i++) {
+    GtkTreeViewColumn *column;
     column = GTK_TREE_VIEW_COLUMN(ptr->data);
     gtk_tree_view_column_set_sort_column_id(column, i);
     gtk_tree_view_column_set_clickable(column, TRUE);
@@ -974,11 +968,11 @@ list_store_select_int(GtkWidget *w, int col, int id)
   GtkTreeIter iter;
   GtkTreeModel *model;
   gboolean state;
-  int val;
 
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(w));
   state = gtk_tree_model_get_iter_first(model, &iter);
   while (state) {
+    int val;
     val = list_store_get_int(w, &iter, col);
     if (val == id) {
       list_store_select_iter(w, &iter);
