@@ -137,7 +137,7 @@ start_editing_enum(GtkCellEditable *editable, struct obj_list_data *d, n_list_st
   g_signal_connect(cbox, "changed", G_CALLBACK(select_enum), d);
 }
 
-#if GTK_CHECK_VERSION(3, 8, 0) && ! GTK_CHECK_VERSION(3, 18, 0)
+#if ! GTK_CHECK_VERSION(3, 18, 0)
 static void
 spin_button_size_allocated(GtkWidget *widget, GdkRectangle *allocation, gpointer user_data)
 {
@@ -197,7 +197,7 @@ start_editing(GtkCellRenderer *renderer, GtkCellEditable *editable, gchar *path,
 	double val;
 
 	getobj(d->obj, list->name, sel, 0, NULL, &val);
-	snprintf(buf, sizeof(buf), "%.15g", val);
+	snprintf(buf, sizeof(buf), DOUBLE_STR_FORMAT, val);
 	gtk_entry_set_text(GTK_ENTRY(editable), buf);
       } else {
 	char *valstr;
@@ -217,7 +217,7 @@ start_editing(GtkCellRenderer *renderer, GtkCellEditable *editable, gchar *path,
   case G_TYPE_DOUBLE:
   case G_TYPE_INT:
     if (GTK_IS_SPIN_BUTTON(editable)) {
-#if GTK_CHECK_VERSION(3, 8, 0) && ! GTK_CHECK_VERSION(3, 18, 0)
+#if ! GTK_CHECK_VERSION(3, 18, 0)
       g_signal_connect(editable, "size-allocate", G_CALLBACK(spin_button_size_allocated), NULL);
 #endif
       gtk_entry_set_alignment(GTK_ENTRY(editable), 1.0);
@@ -497,7 +497,7 @@ set_obj_cell_renderer_cb(struct obj_list_data *d, int i, n_list_store *list, GCa
 void
 update_viewer(struct obj_list_data *d)
 {
-  char *objects[2];
+  char const *objects[2];
   objects[0] = d->obj->name;
   objects[1] = NULL;
   ViewerWinUpdate(objects);
@@ -514,7 +514,7 @@ obj_copy(struct objlist *obj, int dest, int src)
 static void
 copy(struct obj_list_data *d)
 {
-  int sel, id, num, undo;
+  int sel, num;
 
   if (Menulock || Globallock)
     return;
@@ -525,6 +525,7 @@ copy(struct obj_list_data *d)
   num = chkobjlastinst(d->obj);
 
   if (sel >= 0 && sel <= num) {
+    int id, undo;
     undo = menu_save_undo_single(UNDO_TYPE_COPY, d->obj->name);
     id = newobj(d->obj);
     if (id < 0) {
@@ -541,15 +542,15 @@ copy(struct obj_list_data *d)
 static void
 delete(struct obj_list_data *d)
 {
-  int sel, num;
-  int update;
+  int sel, inst_num;
+  int updated;
 
   if (Menulock || Globallock)
     return;
 
   sel = list_store_get_selected_int(GTK_WIDGET(d->text), COL_ID);
-  num = chkobjlastinst(d->obj);
-  if (sel < 0 || sel > num) {
+  inst_num = chkobjlastinst(d->obj);
+  if (sel < 0 || sel > inst_num) {
     return;
   }
 
@@ -562,18 +563,18 @@ delete(struct obj_list_data *d)
     delobj(d->obj, sel);
   }
 
-  num = chkobjlastinst(d->obj);
-  if (num < 0) {
+  inst_num = chkobjlastinst(d->obj);
+  if (inst_num < 0) {
     d->select = -1;
-    update = TRUE;
-  } else if (sel > num) {
-    d->select = num;
-    update = FALSE;
+    updated = TRUE;
+  } else if (sel > inst_num) {
+    d->select = inst_num;
+    updated = FALSE;
   } else {
     d->select = sel;
-    update = FALSE;
+    updated = FALSE;
   }
-  d->update(d, update, TRUE);
+  d->update(d, updated, TRUE);
   set_graph_modified();
 }
 
@@ -666,7 +667,7 @@ move_down(struct obj_list_data *d)
 }
 
 static void
-update(struct obj_list_data *d)
+swin_update(struct obj_list_data *d)
 {
   int sel, ret, num, undo;
   GtkWidget *parent;
@@ -692,7 +693,6 @@ update(struct obj_list_data *d)
     undo = menu_save_undo_single(UNDO_TYPE_EDIT, d->obj->name);
   }
   ret = DialogExecute(parent, d->dialog);
-  set_graph_modified();
   switch (ret) {
   case IDCANCEL:
     menu_undo_internal(undo);
@@ -712,7 +712,7 @@ update(struct obj_list_data *d)
 }
 
 static void
-focus(struct obj_list_data *d, int add)
+focus(struct obj_list_data *d, enum FOCUS_MODE add)
 {
   int sel, num;
 
@@ -726,6 +726,16 @@ focus(struct obj_list_data *d, int add)
     Focus(d->obj, sel, add);
 }
 
+
+static void
+focus_all(struct obj_list_data *d)
+{
+  if (Menulock || Globallock)
+    return;
+
+  ViewerSelectAllObj(d->obj);
+}
+
 static void
 toggle_boolean(struct obj_list_data *d, char *field, int sel)
 {
@@ -831,30 +841,6 @@ hidden(struct obj_list_data *d)
   set_graph_modified();
 }
 
-static void
-set_hidden_state(struct obj_list_data *d, int hide)
-{
-  int sel, num;
-  int hidden;
-
-  if (Menulock || Globallock)
-    return;
-
-  sel = list_store_get_selected_int(GTK_WIDGET(d->text), COL_ID);
-  num = chkobjlastinst(d->obj);
-  if (sel < 0 || sel > num) {
-    return;
-  }
-
-  getobj(d->obj, "hidden", sel, 0, NULL, &hidden);
-  if (hidden != hide) {
-    putobj(d->obj, "hidden", sel, &hide);
-    d->select = sel;
-    d->update(d, FALSE, TRUE);
-    set_graph_modified();
-  }
-}
-
 #if ! GTK_CHECK_VERSION(3, 22, 0)
 static void
 popup_menu_position(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data)
@@ -911,6 +897,138 @@ do_popup(GdkEventButton *event, struct obj_list_data *d)
 #endif
 }
 
+#if GTK_CHECK_VERSION(3, 99, 0)
+static void
+ev_button_down(GtkGestureMultiPress *gesture, gint n_press, gdouble x, gdouble y, gpointer user_data)
+{
+  struct obj_list_data *d;
+#if GTK_CHECK_VERSION(4, 0, 0)
+  static guint32 time = 0;
+  guint32 current_time;
+  int tdif;
+#endif
+  guint button;
+
+  if (Menulock || Globallock) return;
+
+#if GTK_CHECK_VERSION(4, 0, 0)
+  current_time = gtk_event_controller_get_current_event_time(GTK_EVENT_CONTROLLER(gesture));
+  tdif = current_time - time;
+  time = current_time;
+
+  /* following check is necessary for editable column. */
+  if (tdif > 0 && tdif < DOUBLE_CLICK_PERIOD)
+    return;
+#endif
+
+  d = user_data;
+
+  button = gtk_gesture_single_get_current_button(GTK_GESTURE_SINGLE(gesture));
+  switch (button) {
+  case 1:
+    if (n_press > 1) {
+      swin_update(d);
+    }
+    break;
+  }
+}
+
+static void
+ev_button_up(GtkGestureMultiPress *gesture, gint n_press, gdouble x, gdouble y, gpointer user_data)
+{
+  struct obj_list_data *d;
+  guint button;
+
+  if (Menulock || Globallock) return;
+
+  d = user_data;
+
+  button = gtk_gesture_single_get_current_button(GTK_GESTURE_SINGLE(gesture));
+  switch (button) {
+  case 3:
+    if (d->popup) {
+      do_popup(NULL, d);
+    }
+    break;
+  }
+}
+
+static gboolean
+ev_key_down(GtkEventControllerKey *controller, guint keyval, guint keycode, GdkModifierType state, gpointer user_data)
+{
+  struct obj_list_data *d;
+  GtkWidget *w;
+
+  if (Menulock || Globallock)
+    return TRUE;
+
+  d = user_data;
+
+  w = gtk_event_controller_get_widget(GTK_EVENT_CONTROLLER(controller));
+  if (d->ev_key && d->ev_key(w, keyval, state, user_data))
+    return TRUE;
+
+  switch (keyval) {
+  case GDK_KEY_Delete:
+    delete(d);
+    break;
+  case GDK_KEY_Insert:
+    copy(d);
+    break;
+  case GDK_KEY_Home:
+    if (state & GDK_SHIFT_MASK)
+      move_top(d);
+    else
+      return FALSE;
+    break;
+  case GDK_KEY_End:
+    if (state & GDK_SHIFT_MASK)
+      move_last(d);
+    else
+      return FALSE;
+    break;
+  case GDK_KEY_Up:
+    if (state & GDK_SHIFT_MASK)
+      move_up(d);
+    else
+      return FALSE;
+    break;
+  case GDK_KEY_Down:
+    if (state & GDK_SHIFT_MASK)
+      move_down(d);
+    else
+      return FALSE;
+    break;
+  case GDK_KEY_Return:
+    if (state & GDK_SHIFT_MASK) {
+      //      e->state &= ~ GDK_SHIFT_MASK;
+      return FALSE;
+    }
+
+    swin_update(d);
+    break;
+  case GDK_KEY_BackSpace:
+    hidden(d);
+    break;
+  case GDK_KEY_space:
+    if (state & GDK_CONTROL_MASK)
+      return FALSE;
+
+    if (! d->can_focus)
+      return FALSE;
+
+    if (state & GDK_SHIFT_MASK) {
+      list_sub_window_add_focus(NULL, d);
+    } else {
+      list_sub_window_focus(NULL, d);
+    }
+    break;
+  default:
+    return FALSE;
+  }
+  return TRUE;
+}
+#else
 static gboolean
 ev_button_down(GtkWidget *w, GdkEventButton *event,  gpointer user_data)
 {
@@ -935,7 +1053,7 @@ ev_button_down(GtkWidget *w, GdkEventButton *event,  gpointer user_data)
   switch (event->button) {
   case 1:
     if (event->type == GDK_2BUTTON_PRESS) {
-      update(d);
+      swin_update(d);
       return TRUE;
     }
     break;
@@ -983,7 +1101,7 @@ ev_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
   d = user_data;
   e = (GdkEventKey *)event;
 
-  if (d->ev_key && d->ev_key(w, event, user_data))
+  if (d->ev_key && d->ev_key(w, e->keyval, e->state, user_data))
     return TRUE;
 
   switch (e->keyval) {
@@ -1023,7 +1141,7 @@ ev_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
       return FALSE;
     }
 
-    update(d);
+    swin_update(d);
     break;
   case GDK_KEY_BackSpace:
     hidden(d);
@@ -1046,36 +1164,8 @@ ev_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
   }
   return TRUE;
 }
-
-#if WINDOWS
-#include <gdk/gdkwin32.h>
-
-static void
-hide_minimize_menu_item(GtkWidget *widget, gpointer user_data)
-{
-  HWND handle;
-  HMENU menu;
-
-  handle = GDK_WINDOW_HWND(gtk_widget_get_window(widget));
-  menu = GetSystemMenu(handle, FALSE);
-  RemoveMenu(menu, SC_MINIMIZE, MF_BYCOMMAND);
-}
 #endif
 
-gboolean
-focus_in(GtkWidget *widget, GdkEvent *event, gpointer user_data)
-{
-  gtk_grab_add(GTK_WIDGET(widget));
-  return FALSE;
-}
-
-gboolean
-focus_out(GtkWidget *widget, GdkEvent *event, gpointer user_data)
-{
-  gtk_grab_remove(GTK_WIDGET(widget));
-  return FALSE;
-}
-
 static void
 swin_realized(GtkWidget *widget, gpointer user_data)
 {
@@ -1123,11 +1213,37 @@ label_sub_window_create(struct SubWin *d)
 
   swin = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-#if GTK_CHECK_VERSION(3, 8, 0)
   gtk_container_add(GTK_CONTAINER(swin), label);
-#else
-  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin), label);
-#endif
+  d->Win = swin;
+
+  return swin;
+}
+
+GtkWidget *
+parameter_sub_window_create(struct SubWin *d)
+{
+  GtkWidget *grid, *swin;
+  struct obj_list_data *data;
+
+  grid = gtk_grid_new();
+  gtk_grid_set_column_spacing(GTK_GRID(grid), 4);
+
+  data = g_malloc0(sizeof(*data));
+  if (data == NULL) {
+    return NULL;
+  }
+  data->select = -1;
+  data->parent = d;
+  data->undo_save = NULL;
+  data->can_focus = FALSE;
+  data->list = NULL;
+  data->list_col_num = 0;
+  data->text = grid;
+
+  swin = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_container_add(GTK_CONTAINER(swin), grid);
+  d->data.data = data;
   d->Win = swin;
 
   return swin;
@@ -1140,12 +1256,29 @@ list_focused(GtkWidget *widget, GdkEvent *ev, gpointer user_data)
   return FALSE;
 }
 
+#if GTK_CHECK_VERSION(3, 99, 0)
+static void
+add_event_controller(GtkWidget *widget, struct obj_list_data *data)
+{
+  GtkGesture *gesture;
+  GtkEventController *controller;
+
+  gesture = gtk_gesture_multi_press_new(widget);
+  gtk_gesture_single_set_button(GTK_GESTURE_SINGLE(gesture), 0);
+  g_signal_connect(gesture, "pressed", G_CALLBACK(ev_button_down), data);
+  g_signal_connect(gesture, "released", G_CALLBACK(ev_button_up), data);
+
+  controller = gtk_event_controller_key_new(widget);
+  g_signal_connect(controller, "key-pressed", G_CALLBACK(ev_key_down), data);
+}
+#endif
+
 static struct obj_list_data *
 list_widget_create(struct SubWin *d, int lisu_num, n_list_store *list, int can_focus, GtkWidget **w)
 {
   struct obj_list_data *data;
   GtkWidget *lstor, *swin;
-  GList *rend_list, *col_list, *col;
+  GList *col_list, *col;
 
   data = g_malloc0(sizeof(*data));
   data->select = -1;
@@ -1159,9 +1292,13 @@ list_widget_create(struct SubWin *d, int lisu_num, n_list_store *list, int can_f
 
   set_cell_renderer_cb(data, lisu_num, list, lstor);
 
+#if GTK_CHECK_VERSION(3, 99, 0)
+  add_event_controller(lstor, data);
+#else
   g_signal_connect(lstor, "button-press-event", G_CALLBACK(ev_button_down), data);
   g_signal_connect(lstor, "button-release-event", G_CALLBACK(ev_button_up), data);
   g_signal_connect(lstor, "key-press-event", G_CALLBACK(ev_key_down), data);
+#endif
 
   /* to handle key-press-event correctly in single window mode */
   g_signal_connect(lstor, "focus-in-event", G_CALLBACK(list_focused), NULL);
@@ -1175,6 +1312,7 @@ list_widget_create(struct SubWin *d, int lisu_num, n_list_store *list, int can_f
 
   col_list = gtk_tree_view_get_columns(GTK_TREE_VIEW(lstor));
   for (col = g_list_next(col_list); col; col = g_list_next(col)) {
+    GList *rend_list;
     rend_list = gtk_cell_layout_get_cells(GTK_CELL_LAYOUT(col->data));
     gtk_tree_view_column_add_attribute(GTK_TREE_VIEW_COLUMN(col->data),
 				       GTK_CELL_RENDERER(rend_list->data),
@@ -1288,28 +1426,25 @@ list_sub_window_move_down(GtkMenuItem *item, gpointer user_data)
 void
 list_sub_window_update(GtkMenuItem *item, gpointer user_data)
 {
-  update((struct obj_list_data *) user_data);
-}
-
-void
-list_sub_window_hide(GtkMenuItem *item, gpointer user_data)
-{
-  int hide;
-
-  hide = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));
-  set_hidden_state((struct obj_list_data *)user_data, ! hide);
+  swin_update((struct obj_list_data *) user_data);
 }
 
 void
 list_sub_window_focus(GtkMenuItem *item, gpointer user_data)
 {
-  focus((struct obj_list_data *) user_data, FALSE);
+  focus((struct obj_list_data *) user_data, FOCUS_MODE_NORMAL);
+}
+
+void
+list_sub_window_focus_all(GtkMenuItem *item, gpointer user_data)
+{
+  focus_all((struct obj_list_data *) user_data);
 }
 
 void
 list_sub_window_add_focus(GtkMenuItem *item, gpointer user_data)
 {
-  focus((struct obj_list_data *) user_data, TRUE);
+  focus((struct obj_list_data *) user_data, FOCUS_MODE_TOGGLE);
 }
 
 static gboolean
