@@ -1,11 +1,11 @@
+#include "gtk_common.h"
+
 #include <stdlib.h>
 #include <math.h>
 
 #include "mathfn.h"
 #include "object.h"
-#include "otext.h"
 
-#include "gtk_common.h"
 #include "gtk_widget.h"
 #include "ox11menu.h"
 #include "x11menu.h"
@@ -15,17 +15,11 @@
 void
 set_button_icon(GtkWidget *w, const char *icon_name)
 {
-#if ! GTK_CHECK_VERSION(3, 12, 0)
-  GtkWidget *icon;
-  icon = gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_BUTTON);
-  gtk_button_set_image(GTK_BUTTON(w), icon);
-#endif
 }
 
 void
 set_widget_margin(GtkWidget *w, int margin_pos)
 {
-#if GTK_CHECK_VERSION(3, 12, 0)
   if (margin_pos & WIDGET_MARGIN_LEFT) {
     gtk_widget_set_margin_start(w, 4);
   }
@@ -33,15 +27,6 @@ set_widget_margin(GtkWidget *w, int margin_pos)
   if (margin_pos & WIDGET_MARGIN_RIGHT) {
     gtk_widget_set_margin_end(w, 4);
   }
-#else
-  if (margin_pos & WIDGET_MARGIN_LEFT) {
-    gtk_widget_set_margin_left(w, 4);
-  }
-
-  if (margin_pos & WIDGET_MARGIN_RIGHT) {
-    gtk_widget_set_margin_right(w, 4);
-  }
-#endif
 
   if (margin_pos & WIDGET_MARGIN_BOTTOM) {
     gtk_widget_set_margin_bottom(w, 4);
@@ -306,7 +291,7 @@ create_file_entry(struct objlist *obj)
 static void
 direction_icon_released(GtkEntry *entry, GtkEntryIconPosition pos, GdkEvent *event, gpointer user_data)
 {
-  int angle, val, rest;
+  int angle, val;
 
   angle = gtk_spin_button_get_value(GTK_SPIN_BUTTON(entry));
   val = angle % 360;
@@ -325,6 +310,7 @@ direction_icon_released(GtkEntry *entry, GtkEntryIconPosition pos, GdkEvent *eve
     if (angle == 0) {
       val = 360;
     } else {
+      int rest;
       rest = val % 90;
       if (rest == 0) {
 	val -= 90;
@@ -347,9 +333,7 @@ create_direction_entry(void)
 
   w = create_spin_entry_type(SPIN_BUTTON_TYPE_ANGLE, FALSE, TRUE);
   gtk_entry_set_width_chars(GTK_ENTRY(w), NUM_ENTRY_WIDTH);
-#if GTK_CHECK_VERSION(3, 12, 0)
   gtk_entry_set_max_width_chars(GTK_ENTRY(w), NUM_ENTRY_WIDTH);
-#endif
   gtk_entry_set_icon_from_icon_name(GTK_ENTRY(w), GTK_ENTRY_ICON_SECONDARY, "go-up-symbolic");
   gtk_entry_set_icon_from_icon_name(GTK_ENTRY(w), GTK_ENTRY_ICON_PRIMARY, "go-down-symbolic");
   g_signal_connect(w, "icon-release", G_CALLBACK(direction_icon_released), NULL);
@@ -357,8 +341,6 @@ create_direction_entry(void)
   return w;
 }
 
-
-
 GtkWidget *
 create_text_entry(int set_default_size, int set_default_action)
 {
@@ -376,6 +358,15 @@ create_text_entry(int set_default_size, int set_default_action)
   return w;
 }
 
+GtkWidget *
+create_number_entry(int set_default_size, int set_default_action)
+{
+  GtkWidget *w;
+  w = create_text_entry(set_default_size, set_default_action);
+  gtk_entry_set_input_purpose(GTK_ENTRY(w), GTK_INPUT_PURPOSE_NUMBER);
+  return w;
+}
+
 static double
 int2val(enum SPIN_BUTTON_TYPE type, int ival)
 {
@@ -525,9 +516,7 @@ _create_spin_entry(enum SPIN_BUTTON_TYPE type, double min, double max,
 
   if (set_default_size) {
     gtk_entry_set_width_chars(GTK_ENTRY(w), NUM_ENTRY_WIDTH);
-#if GTK_CHECK_VERSION(3, 12, 0)
     gtk_entry_set_max_width_chars(GTK_ENTRY(w), NUM_ENTRY_WIDTH);
-#endif
   }
 
   if (set_default_action) {
@@ -1081,3 +1070,49 @@ select_obj_color(struct objlist *obj, int id, enum OBJ_FIELD_COLOR_TYPE type)
   }
   return modified;
 }
+
+gchar *
+get_text_from_buffer(GtkTextBuffer *buffer)
+{
+  GtkTextIter start, end;
+  gtk_text_buffer_get_start_iter(buffer, &start);
+  gtk_text_buffer_get_end_iter(buffer, &end);
+  return gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
+}
+
+static void
+add_button_common(GtkWidget *w, GtkWidget *grid, int row, int col, const char *tooltip, GCallback proc, gpointer data)
+{
+  gtk_widget_set_tooltip_text(GTK_WIDGET(w), tooltip);
+  gtk_widget_set_vexpand(GTK_WIDGET(w), FALSE);
+  gtk_widget_set_valign(GTK_WIDGET(w), GTK_ALIGN_CENTER);
+  gtk_widget_set_hexpand(GTK_WIDGET(w), FALSE);
+  gtk_widget_set_halign(GTK_WIDGET(w), GTK_ALIGN_START);
+  gtk_grid_attach(GTK_GRID(grid), w, col, row, 1, 1);
+  if (proc) {
+    g_signal_connect(w, "clicked", proc, data);
+  }
+}
+
+GtkWidget *
+add_button(GtkWidget *grid, int row, int col, const char *icon, const char *tooltip, GCallback proc, gpointer data)
+{
+  GtkWidget *w;
+  w = gtk_button_new_from_icon_name(icon, GTK_ICON_SIZE_BUTTON);
+  add_button_common(w, grid, row, col, tooltip, proc, data);
+  return w;
+}
+
+GtkWidget *
+add_toggle_button(GtkWidget *grid, int row, int col, const char *icon_name, const char *tooltip, GCallback proc, gpointer data)
+{
+  GtkWidget *w, *icon;
+  w = gtk_toggle_button_new();
+  icon = gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_BUTTON);
+  gtk_button_set_image(GTK_BUTTON(w), icon);
+  add_button_common(w, grid, row, col, tooltip, NULL, NULL);
+  if (proc) {
+    g_signal_connect(w, "toggled", proc, data);
+  }
+  return w;
+}
