@@ -60,6 +60,7 @@ int entry_set_filename(GtkWidget *w, char *filename);
 
 GtkWidget *create_color_button(GtkWidget *win);
 GtkWidget *create_text_entry(int set_default_size, int set_default_action);
+GtkWidget *create_number_entry(int set_default_size, int set_default_action);
 GtkWidget *create_file_entry(struct objlist *obj);
 GtkWidget *create_file_entry_with_cb(GCallback cb, gpointer data);
 GtkWidget *create_direction_entry(void);
@@ -83,6 +84,8 @@ void set_widget_font(GtkWidget *w, const char *font);
 #endif
 void add_default_color(struct narray *palette);
 void add_default_gray(struct narray *palette);
-
+gchar *get_text_from_buffer(GtkTextBuffer *buffer);
+GtkWidget *add_button(GtkWidget *grid, int row, int col, const char *icon, const char *tooltip, GCallback proc, gpointer data);
+GtkWidget *add_toggle_button(GtkWidget *grid, int row, int col, const char *icon_name, const char *tooltip, GCallback proc, gpointer data);
 
 #endif
