@@ -895,6 +895,7 @@ n_initialize(int *argc, char ***argv)
 #ifdef HAVE_READLINE_READLINE_H
   int history_size = HIST_SIZE;
 #endif
+  struct lconv *lconv;
 
   if (Initialized) {
     return 0;
@@ -956,6 +957,10 @@ n_initialize(int *argc, char ***argv)
   locale = setlocale(LC_ALL, NULL);
   locale = g_strdup(CHK_STR(locale));
 #endif /* WINDOWS */
+  /* to use a period as a decimal separator */
+  lconv = localeconv();
+  set_system_decimalsign(lconv->decimal_point);
+  setlocale(LC_NUMERIC, "C");
 
   libdir = g_strdup(LIBDIR);
   if (libdir == NULL)
