@@ -93,6 +93,9 @@ struct gtklocal
   struct action_type action;
   double bg_r, bg_g, bg_b;
   struct gra2cairo_local *local;
+#if OSX
+  int menulock;
+#endif
 };
 
 static void gtkMakeRuler(cairo_t *cr, struct gtklocal *gtklocal);
@@ -126,7 +129,7 @@ gtkevpaint(GtkWidget *w, cairo_t *cr, gpointer user_data)
   }
 
   if (gtklocal->redraw) {
-    GRAredraw(gtklocal->obj, gtklocal->inst, FALSE, FALSE);
+    GRAredraw(gtklocal->obj, gtklocal->inst, TRUE, 0);
     gtklocal->redraw = FALSE;
   }
 
@@ -140,13 +143,13 @@ gtkevpaint(GtkWidget *w, cairo_t *cr, gpointer user_data)
 static int
 gtkclose(GtkWidget *widget, GdkEvent *event, gpointer user_data)
 {
-  N_VALUE *inst;
   struct gtklocal *local;
   int i, id;
   struct objlist *obj;
 
   obj = chkobject("gra2gtk");
   for (i = 0; i <= chkobjlastinst(obj); i++) {
+    N_VALUE *inst;
     inst = chkobjinst(obj, i);
     _getobj(obj, "_gtklocal", inst, &local);
     if (local->mainwin == widget) {
@@ -158,6 +161,28 @@ gtkclose(GtkWidget *widget, GdkEvent *event, gpointer user_data)
   return TRUE;
 }
 
+#if USE_EVENT_CONTROLLER
+static gboolean
+ev_key_down(GtkEventControllerKey *controller, guint keyval, guint keycode, GdkModifierType state, gpointer user_data)
+{
+  struct gtklocal *gtklocal;
+
+  gtklocal = (struct gtklocal *) user_data;
+
+  gtklocal->action.type = ACTION_TYPE_KEY;
+  gtklocal->action.val = keyval;
+
+  switch (keyval) {
+  case GDK_KEY_w:
+    if (state & GDK_CONTROL_MASK) {
+      gtkclose(gtklocal->mainwin, NULL, NULL);
+      return TRUE;
+    }
+    return FALSE;
+  }
+  return FALSE;
+}
+#else
 static gboolean
 ev_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
 {
@@ -184,6 +209,7 @@ ev_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
   }
   return FALSE;
 }
+#endif
 
 void
 size_allocate(GtkWidget *widget, GdkRectangle *allocation, gpointer user_data)
@@ -227,8 +253,13 @@ size_allocate(GtkWidget *widget, GdkRectangle *allocation, gpointer user_data)
 
 }
 
+#if USE_EVENT_CONTROLLER
+static void
+cursor_moved(GtkEventControllerMotion *controller, gdouble x, gdouble y, gpointer user_data)
+#else
 static gboolean
 cursor_moved(GtkWidget *widget, GdkEvent  *event, gpointer user_data)
+#endif
 {
   struct gtklocal *gtklocal;
 
@@ -240,9 +271,25 @@ cursor_moved(GtkWidget *widget, GdkEvent  *event, gpointer user_data)
     gtklocal->blank_cursor = NULL;
   }
 
+#if ! USE_EVENT_CONTROLLER
   return FALSE;
+#endif
 }
 
+#if USE_EVENT_CONTROLLER
+static void
+button_released(GtkGestureMultiPress *gesture, gint n_press, gdouble x, gdouble y, gpointer user_data)
+{
+  struct gtklocal *gtklocal;
+  guint button;
+
+  gtklocal = (struct gtklocal *) user_data;
+  button = gtk_gesture_single_get_current_button(GTK_GESTURE_SINGLE(gesture));
+
+  gtklocal->action.type = ACTION_TYPE_BUTTON;
+  gtklocal->action.val = button;
+}
+#else
 static gboolean
 button_released(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
@@ -255,6 +302,7 @@ button_released(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 
   return FALSE;
 }
+#endif
 
 static gboolean
 scrolled(GtkWidget *widget, GdkEventScroll *event, gpointer user_data)
@@ -278,6 +326,35 @@ scrolled(GtkWidget *widget, GdkEventScroll *event, gpointer user_data)
   return FALSE;
 }
 
+#if USE_EVENT_CONTROLLER
+static void
+add_event_key(GtkWidget *widget, struct gtklocal *gtklocal)
+{
+  GtkEventController *ev;
+
+  ev = gtk_event_controller_key_new(widget);
+  g_signal_connect(ev, "key-pressed", G_CALLBACK(ev_key_down), gtklocal);
+}
+
+static void
+add_event_button(GtkWidget *widget, struct gtklocal *gtklocal)
+{
+  GtkGesture *ev;
+  ev = gtk_gesture_multi_press_new(widget);
+  gtk_gesture_single_set_button(GTK_GESTURE_SINGLE(ev), 0);
+  g_signal_connect(ev, "released", G_CALLBACK(button_released), gtklocal);
+}
+
+static void
+add_event_motion(GtkWidget *widget, struct gtklocal *gtklocal)
+{
+  GtkEventController *ev;
+
+  ev = gtk_event_controller_motion_new(widget);
+  g_signal_connect(ev, "motion", G_CALLBACK(cursor_moved), gtklocal);
+}
+#endif
+
 static int
 gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
@@ -326,6 +403,9 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   gtklocal->bg_g = 1.0;
   gtklocal->bg_b = 1.0;
   gtklocal->local = local;
+#if OSX
+  gtklocal->menulock = FALSE;
+#endif
 
   if (gtklocal->windpi < 1)
     gtklocal->windpi = 1;
@@ -362,12 +442,24 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   gtklocal->title = mkobjlist(obj, NULL, oid, NULL, TRUE);
 
   gtklocal->mainwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  if (TopLevel) {
+    gtk_window_set_modal(GTK_WINDOW(gtklocal->mainwin), TRUE);
+    gtk_window_set_transient_for(GTK_WINDOW(gtklocal->mainwin), GTK_WINDOW(TopLevel));
+#if OSX
+    gtklocal->menulock = Menulock;
+    Menulock = TRUE;
+#endif
+  }
   gtk_window_set_default_size(GTK_WINDOW(gtklocal->mainwin), width, height);
   g_signal_connect_swapped(gtklocal->mainwin,
 			   "delete-event",
 			   G_CALLBACK(gtkclose), gtklocal->mainwin);
 
+#if USE_EVENT_CONTROLLER
+  add_event_key(gtklocal->mainwin, gtklocal);
+#else
   g_signal_connect(gtklocal->mainwin, "key-press-event", G_CALLBACK(ev_key_down), gtklocal);
+#endif
 
   gtk_window_set_title((GtkWindow *) gtklocal->mainwin, gtklocal->title);
 
@@ -385,12 +477,7 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   g_signal_connect(gtklocal->mainwin, "size-allocate",
 		   G_CALLBACK(size_allocate), gtklocal);
 
-#if GTK_CHECK_VERSION(3, 8, 0)
   gtk_container_add(GTK_CONTAINER(scrolled_window), gtklocal->View);
-#else
-  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW
-					(scrolled_window), gtklocal->View);
-#endif
 
   gtk_widget_show_all(gtklocal->mainwin);
 
@@ -398,8 +485,13 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   gtklocal->redraw = TRUE;
 
   gtk_widget_add_events(gtklocal->mainwin, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK);
+#if USE_EVENT_CONTROLLER
+  add_event_motion(gtklocal->mainwin, gtklocal);
+  add_event_button(gtklocal->mainwin, gtklocal);
+#else
   g_signal_connect(gtklocal->mainwin, "motion-notify-event", G_CALLBACK(cursor_moved), gtklocal);
   g_signal_connect(gtklocal->mainwin, "button-release-event", G_CALLBACK(button_released), gtklocal);
+#endif
   g_signal_connect(gtklocal->mainwin, "scroll-event", G_CALLBACK(scrolled), gtklocal);
 
   if (chkobjfield(obj, "_evloop")) {
@@ -471,6 +563,11 @@ gtkdone(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
 
   g_free(gtklocal->title);
 
+#if OSX
+  if (TopLevel) {
+    Menulock = gtklocal->menulock;
+  }
+#endif
   return 0;
 }
 
@@ -592,40 +689,27 @@ get_color(struct gtklocal *gtklocal, int argc, char **argv)
   return c / 255.0;
 }
 
-
-static int
-gtkbr(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
-{
-  struct gtklocal *gtklocal;
-
-  if (_getobj(obj, "_gtklocal", inst, &gtklocal))
-    return 1;
-
-  gtklocal->bg_r = get_color(gtklocal, argc, argv);
-  return 0;
-}
-
-static int
-gtkbb(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
-{
-  struct gtklocal *gtklocal;
-
-  if (_getobj(obj, "_gtklocal", inst, &gtklocal))
-    return 1;
-
-  gtklocal->bg_b = get_color(gtklocal, argc, argv);
-  return 0;
-}
-
 static int
 gtkbg(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   struct gtklocal *gtklocal;
+  const char *field;
 
   if (_getobj(obj, "_gtklocal", inst, &gtklocal))
     return 1;
 
-  gtklocal->bg_g = get_color(gtklocal, argc, argv);
+  field = argv[1];
+  switch(field[1]) {
+  case 'R':
+    gtklocal->bg_r = get_color(gtklocal, argc, argv);
+    break;
+  case 'G':
+    gtklocal->bg_g = get_color(gtklocal, argc, argv);
+    break;
+  case 'B':
+    gtklocal->bg_b = get_color(gtklocal, argc, argv);
+    break;
+  }
   return 0;
 }
 
@@ -919,9 +1003,9 @@ static struct objtable gra2gtk[] = {
   {"clear", NVFUNC, NREAD | NEXEC, gtkclear, "", 0},
   {"present", NVFUNC, NREAD | NEXEC, gtkpresent, "", 0},
   {"fullscreen", NVFUNC, NREAD | NEXEC, gtkfullscreen, "b", 0},
-  {"BR", NINT, NREAD | NWRITE, gtkbr, NULL, 0},
+  {"BR", NINT, NREAD | NWRITE, gtkbg, NULL, 0},
   {"BG", NINT, NREAD | NWRITE, gtkbg, NULL, 0},
-  {"BB", NINT, NREAD | NWRITE, gtkbb, NULL, 0},
+  {"BB", NINT, NREAD | NWRITE, gtkbg, NULL, 0},
   {"wait_action",NSFUNC, NREAD | NEXEC, gtkwait_action, "", 0},
   {"_gtklocal", NPOINTER, 0, NULL, NULL, 0},
   {"_output", NVFUNC, 0, gtk_output, NULL, 0},
