@@ -23,10 +23,14 @@
 
 char *AxisCB(struct objlist *obj, int id);
 
-void CmAxisNewFrame(void *w, gpointer client_data);
-void CmAxisNewSection(void *w, gpointer client_data);
-void CmAxisNewCross(void *w, gpointer client_data);
-void CmAxisNewSingle(void *w, gpointer client_data);
+void CmAxisNewFrame(int use_presettings);
+void CmAxisNewSection(int use_presettings);
+void CmAxisNewCross(int use_presettings);
+
+void CmAxisAddFrame(void *w, gpointer client_data);
+void CmAxisAddSection(void *w, gpointer client_data);
+void CmAxisAddCross(void *w, gpointer client_data);
+void CmAxisAddSingle(void *w, gpointer client_data);
 
 void CmAxisUpdate(void *w, gpointer client_data);
 void CmAxisDel(void *w, gpointer client_data);
@@ -38,8 +42,9 @@ void CmAxisGridNew(void *w, gpointer client_data);
 void CmAxisGridDel(void *w, gpointer client_data);
 void CmAxisGridUpdate(void *w, gpointer client_data);
 
-GtkWidget *dreate_axis_list(struct SubWin *d);
+GtkWidget *create_axis_list(struct SubWin *d);
 
 void axis_scale_push(struct objlist *obj, int id);
 void AxisWinUpdate(struct obj_list_data *data, int clear, int draw);
 int axis_check_history(void);
+void update_viewer_axisgrid(void);
