@@ -33,7 +33,6 @@
 #define NGRAPH_ALIGN_T_ICON	"ngraph_align_t"
 #define NGRAPH_ALIGN_VC_ICON	"ngraph_align_vc"
 #define NGRAPH_MATH_ICON	"ngraph_math"
-#define NGRAPH_UNDO_ICON	"ngraph_undo"
 
 #define NGRAPH_SVG_ICON_FILE		RESOURCE_PATH "/pixmaps/ngraph.svg"
 #define NGRAPH_ICON_FILE		RESOURCE_PATH "/pixmaps/ngraph_icon.png"
@@ -61,3 +60,6 @@
 #define NGRAPH_AXISWIN_ICON	"ngraph_axiswin-symbolic"
 #define NGRAPH_FILEWIN_ICON	"ngraph_filewin-symbolic"
 #define NGRAPH_MERGEWIN_ICON	"ngraph_mergewin-symbolic"
+#define NGRAPH_PARAMETER_ICON	"ngraph_parameter-symbolic"
+#define NGRAPH_EXCHANGE_ICON	"ngraph_exchange-symbolic"
+#define NGRAPH_LINK_ICON	"ngraph_link-symbolic"
