@@ -99,6 +99,7 @@ OpenGRA(void)
     putobj(Menulocal.GRAobj, "left_margin", id, &(Menulocal.LeftMargin));
     putobj(Menulocal.GRAobj, "top_margin", id, &(Menulocal.TopMargin));
     putobj(Menulocal.GRAobj, "zoom", id, &(Menulocal.PaperZoom));
+    putobj(Menulocal.GRAobj, "decimalsign", id, &(Menulocal.Decimalsign));
     CheckPage();
   } else {
     /* find closed gra object */
@@ -148,6 +149,8 @@ CheckPage(void)
 	  &(Menulocal.TopMargin));
   _getobj(Menulocal.GRAobj, "zoom", gra_inst,
 	  &(Menulocal.PaperZoom));
+  _getobj(Menulocal.GRAobj, "decimalsign", gra_inst,
+	  &(Menulocal.Decimalsign));
 
   set_paper_type(Menulocal.PaperWidth, Menulocal.PaperHeight);
 }
@@ -238,6 +241,7 @@ SetPageSettingsToGRA(void)
     putobj(obj, "left_margin", id, &(Menulocal.LeftMargin));
     putobj(obj, "top_margin", id, &(Menulocal.TopMargin));
     putobj(obj, "zoom", id, &(Menulocal.PaperZoom));
+    putobj(obj, "decimalsign", id, &(Menulocal.Decimalsign));
 
     _getobj(obj, "draw_obj", inst, &drawrable);
 
@@ -891,6 +895,8 @@ DeleteDrawable(void)
     delete_instances(prmobj);
   }
   set_toolbox_mode(TOOLBOX_MODE_TOOLBAR);
+  Menulocal.Decimalsign = get_gra_decimalsign_type(Menulocal.default_decimalsign);
+  gra_set_default_decimalsign(Menulocal.Decimalsign);
 }
 
 static void
@@ -1115,7 +1121,7 @@ SaveParent(int hFile, struct objlist *parent, int storedata,
 }
 
 int
-SaveDrawrable(char *name, int storedata, int storemerge)
+SaveDrawrable(char *name, int storedata, int storemerge, int save_decimalsign)
 {
   int hFile;
   int error;
@@ -1159,6 +1165,10 @@ SaveDrawrable(char *name, int storedata, int storemerge)
 	  arrayinit(&sarray, sizeof(char *));
 	  opt = "device";
 	  arrayadd(&sarray, &opt);
+	  if (! save_decimalsign) {
+	    opt = "decimalsign";
+	    arrayadd(&sarray, &opt);
+	  }
 	  arg[0] = (char *) &sarray;
 	  arg[1] = NULL;
 	  getobj(graobj, "save", id, 1, arg, &s);
@@ -1274,7 +1284,7 @@ GraphSave(int overwrite)
       char mes[256];
       snprintf(mes, sizeof(mes), _("Saving `%.128s'."), file);
       SetStatusBar(mes);
-      if(SaveDrawrable(file, sdata, smerge)) {
+      if(SaveDrawrable(file, sdata, smerge, TRUE)) {
 	ret = IDCANCEL;
       } else {
 	switch (path) {
@@ -1614,7 +1624,7 @@ FileAutoScale(void)
   if (lastinst < 0 || aobj == NULL || anum == 0)
     return;
 
-  str = g_string_new("file:");
+  str = g_string_new("data:");
   if (str == NULL) {
     error(NULL, ERRHEAP);
     return;
