@@ -391,7 +391,7 @@ struct PageDialog
 {
   DIALOG_PROTOTYPE;
   /****** local member *******/
-  GtkWidget *leftmargin, *topmargin, *paperwidth, *paperheight, *paperzoom, *paper;
+  GtkWidget *leftmargin, *topmargin, *paperwidth, *paperheight, *paperzoom, *paper, *decimalsign;
 };
 void PageDialog(struct PageDialog *data);
 
@@ -495,7 +495,8 @@ struct MiscDialog
   GtkWidget *editor, *directory, *path, *datafile,
     *expand, *expanddir, *loadpath, *mergefile, *coordwin_font, *infowin_font,
     *file_preview_font, *hist_size, *info_size, *data_head_lines, *help_browser,
-    *browser, *use_opacity, *select_data, *use_custom_palette, *source_style;
+    *browser, *use_opacity, *select_data, *use_custom_palette, *source_style,
+    *decimalsign;
   struct objlist *Obj;
   struct narray tmp_palette;
   int Id;
