@@ -134,6 +134,7 @@ PageDialogSetupItem(GtkWidget *w, struct PageDialog *d)
   j = set_paper_type(Menulocal.PaperWidth, Menulocal.PaperHeight);
 
   combo_box_set_active(d->paper, j);
+  combo_box_set_active(d->decimalsign, Menulocal.Decimalsign);
 }
 
 static void
@@ -204,6 +205,14 @@ PageDialogSetup(GtkWidget *wi, void *data, int makewidget)
     add_widget_to_table(table, w, _("paper _Zoom:"), FALSE, i++);
     d->paperzoom = w;
 
+    w = combo_box_create();
+    gtk_widget_set_hexpand(w, TRUE);
+    add_widget_to_table(table, w, _("_Decimalsign:"), FALSE, i++);
+    for (j = 0; gra_decimalsign_char[j]; j++) {
+      combo_box_append_text(w, _(gra_decimalsign_char[j]));
+    }
+    d->decimalsign = w;
+
     gtk_box_pack_start(GTK_BOX(d->vbox), table, FALSE, FALSE, 4);
     gtk_widget_show_all(GTK_WIDGET(d->vbox));
   }
@@ -234,6 +243,7 @@ PageDialogClose(GtkWidget *wi, void *data)
   Menulocal.TopMargin = spin_entry_get_val(d->topmargin);
 
   Menulocal.PaperZoom = spin_entry_get_val(d->paperzoom);
+  Menulocal.Decimalsign = combo_box_get_active(d->decimalsign);
 }
 
 void
@@ -733,7 +743,7 @@ SwitchDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 4);
     d->bottom = w;
 
-    w = gtk_button_new_with_mnemonic(("_Remove"));
+    w = gtk_button_new_with_mnemonic(_("_Remove"));
     set_button_icon(w, "list-remove");
     g_signal_connect(w, "clicked", G_CALLBACK(SwitchDialogRemove), d);
     gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 4);
@@ -869,7 +879,7 @@ DirectoryDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_widget_set_hexpand(w, TRUE);
     gtk_widget_set_halign(w, GTK_ALIGN_START);
     g_object_set(w, "margin", GINT_TO_POINTER(4), NULL);
-    gtk_grid_attach(GTK_GRID(table), w, 1, 2, 1, 1);
+    gtk_grid_attach(GTK_GRID(table), w, 1, 1, 1, 1);
 
     gtk_box_pack_start(GTK_BOX(d->vbox), table, FALSE, FALSE, 4);
     gtk_widget_show_all(GTK_WIDGET(d->vbox));
