@@ -44,6 +44,7 @@ enum MenuID {
   MenuIdEditPaste,
   MenuIdEditDelete,
   MenuIdEditDuplicate,
+  MenuIdEditSelectAll,
   MenuIdAlignLeft,
   MenuIdAlignVCenter,
   MenuIdAlignRight,
@@ -128,6 +129,18 @@ enum MouseMode {
   MOUSESCROLLE,
 };
 
+enum PointerMode {
+  PointerModeBoth,
+  PointerModeLegend,
+  PointerModeAxis,
+  PointerModeData,
+  PointerModeNum,
+  PointerModeDefault,
+  PointerModeFocus,
+  PointerModeFocusAxis,
+  PointerModeFocusLegend,
+};
+
 enum pop_up_menu_item_type {
   POP_UP_MENU_ITEM_TYPE_NORMAL,
   POP_UP_MENU_ITEM_TYPE_CHECK,
@@ -138,11 +151,33 @@ enum pop_up_menu_item_type {
   POP_UP_MENU_ITEM_TYPE_END,
 };
 
+struct ScrollPrm {
+  double x, y;
+};
+
+struct ZoomPrm {
+  int x, y, dpi;
+  int focused;
+  double scale;
+};
+
+struct DragPrm {
+  int x, y;
+  int active;
+  double vx, vy;
+};
+
+struct DecelerationPrm
+{
+  guint id;
+  guint64 start;
+};
+
 #define VIEWER_POPUP_ITEM_NUM 14
 struct Viewer
 {
   GtkWidget *Win;
-  GtkWidget *menu, *VScroll, *HScroll, *popup, *VRuler, *HRuler, *side_pane1, *side_pane2, *side_pane3, *main_pane;
+  GtkWidget *VScroll, *HScroll, *popup, *VRuler, *HRuler, *side_pane1, *side_pane2, *side_pane3, *main_pane;
   int ShowFrame, ShowLine, ShowRect;
   int Capture, MoveData, KeyMask;
   enum MouseMode MouseMode;
@@ -151,11 +186,14 @@ struct Viewer
   int FrameOfsX, FrameOfsY;
   int MouseX1, MouseY1, MouseX2, MouseY2, MouseDX, MouseDY;
   int RefX1, RefY1, RefX2, RefY2, ChangePoint;
-  int LineX, LineY, CrossX, CrossY, Angle;
-  int allclear;
+  int LineX, LineY, Angle;
   int cx, cy;
   int ignoreredraw;
-  double vscroll, hscroll, ZoomX, ZoomY;
+  double vscroll, hscroll, ZoomX, ZoomY, CrossX, CrossY;
+  struct ZoomPrm zoom_prm;
+  struct DragPrm drag_prm;
+  struct DecelerationPrm deceleration_prm;
+  struct ScrollPrm scroll_prm;
 };
 
 enum SubWinType {
@@ -167,6 +205,7 @@ enum SubWinType {
   TypeMarkWin,
   TypeTextWin,
   TypeMergeWin,
+  TypeParameterWin,
   TypeCoordWin,
   TypeInfoWin,
 };
@@ -183,7 +222,7 @@ struct obj_list_data
   void (* setup_dialog)(struct obj_list_data *data, int id, int user_data);
   int (* undo_save)(int type);
   void *dialog;
-  gboolean (* ev_key) (GtkWidget *, GdkEvent *, gpointer);
+  gboolean (* ev_key) (GtkWidget *, guint, GdkModifierType, gpointer);
   struct objlist *obj;
   struct SubWin *parent;
   n_list_store *list;
@@ -222,6 +261,7 @@ struct NgraphApp
   struct SubWin MarkWin;
   struct SubWin TextWin;
   struct SubWin MergeWin;
+  struct SubWin ParameterWin;
   struct SubWin CoordWin;
   struct SubWin InfoWin;
 };
@@ -232,9 +272,7 @@ extern struct NgraphApp NgraphApp;
 extern GtkWidget *TopLevel;
 extern GdkColor white, gray;
 extern GtkAccelGroup *AccelGroup;
-#if USE_GTK_BUILDER
 extern GtkApplication *GtkApp;
-#endif
 
 enum MENU_UNDO_TYPE {
   UNDO_TYPE_EDIT,
@@ -272,6 +310,14 @@ enum RerawFlag {
   DRAW_AXIS_ONLY = 4,
 };
 
+enum TOOLBOX_MODE {
+  TOOLBOX_MODE_TOOLBAR,
+  TOOLBOX_MODE_SETTING_PANEL,
+};
+
+void set_toolbox_mode(enum TOOLBOX_MODE mode);
+enum TOOLBOX_MODE get_toolbox_mode(void);
+
 int application(char *file);
 
 void set_current_window(GtkWidget *w);
