@@ -71,6 +71,7 @@ static struct subwin_popup_list Popup_list[] = {
   {N_("_Delete"),       G_CALLBACK(list_sub_window_delete), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {NULL, NULL, NULL, POP_UP_MENU_ITEM_TYPE_SEPARATOR},
   {"_Focus",           G_CALLBACK(list_sub_window_focus), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
+  {N_("focus _All"),   G_CALLBACK(list_sub_window_focus_all), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {N_("_Preferences"), G_CALLBACK(list_sub_window_update), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {N_("_Instance name"), G_CALLBACK(list_sub_window_object_name), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {NULL, NULL, NULL, POP_UP_MENU_ITEM_TYPE_SEPARATOR},
@@ -82,15 +83,17 @@ static struct subwin_popup_list Popup_list[] = {
 };
 
 #define POPUP_ITEM_NUM (sizeof(Popup_list) / sizeof(*Popup_list) - 1)
-#define POPUP_ITEM_TOP     8
-#define POPUP_ITEM_UP      9
-#define POPUP_ITEM_DOWN   10
-#define POPUP_ITEM_BOTTOM 11
+#define POPUP_ITEM_FOCUS_ALL 5
+#define POPUP_ITEM_TOP       9
+#define POPUP_ITEM_UP       10
+#define POPUP_ITEM_DOWN     11
+#define POPUP_ITEM_BOTTOM   12
 
 
 static void
 MergeDialogSetupItem(struct MergeDialog *d, int file, int id)
 {
+  double zm_x, zm_y;
   if (file) {
     SetWidgetFromObjField(d->file, d->Obj, id, "file");
     gtk_editable_set_position(GTK_EDITABLE(d->file), -1);
@@ -99,6 +102,9 @@ MergeDialogSetupItem(struct MergeDialog *d, int file, int id)
   SetWidgetFromObjField(d->leftmargin, d->Obj, id, "left_margin");
   SetWidgetFromObjField(d->zoom_x, d->Obj, id, "zoom_x");
   SetWidgetFromObjField(d->zoom_y, d->Obj, id, "zoom_y");
+  zm_x = gtk_spin_button_get_value(GTK_SPIN_BUTTON(d->zoom_x));
+  zm_y = gtk_spin_button_get_value(GTK_SPIN_BUTTON(d->zoom_y));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->link), zm_x == zm_y);
 }
 
 static void
@@ -109,19 +115,45 @@ MergeDialogCopy(GtkWidget *w, gpointer data)
 
   d = (struct MergeDialog *) data;
 
-  sel = CopyClick(d->widget, d->Obj, d->Id, FileCB);
+  sel = CopyClick(d->widget, d->Obj, d->Id, MergeFileCB);
   if (sel != -1) {
     MergeDialogSetupItem(d, FALSE, sel);
   }
 }
 
+static void
+zoom_changed(GtkSpinButton *spin_button, gpointer user_data)
+{
+  struct MergeDialog *d;
+  int linked;
+  double zoom;
+  d = user_data;
+  linked = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->link));
+  if (! linked) {
+    return;
+  }
+  zoom = gtk_spin_button_get_value(spin_button);
+  gtk_spin_button_set_value(GTK_SPIN_BUTTON(d->zoom_y), zoom);
+}
+
+static void
+link_toggled(GtkToggleButton *button, gpointer user_data)
+{
+  struct MergeDialog *d;
+  int linked;
+  d = user_data;
+  linked = gtk_toggle_button_get_active(button);
+  gtk_widget_set_sensitive(d->zoom_y, ! linked);
+  if (linked) {
+    zoom_changed(GTK_SPIN_BUTTON(d->zoom_x), user_data);
+  }
+}
+
 static void
 MergeDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *frame, *table;
   struct MergeDialog *d;
   char title[64];
-  int i;
 
   d = (struct MergeDialog *) data;
 
@@ -129,6 +161,8 @@ MergeDialogSetup(GtkWidget *wi, void *data, int makewidget)
   gtk_window_set_title(GTK_WINDOW(wi), title);
 
   if (makewidget) {
+    GtkWidget *w, *frame, *table;
+    int i;
     table = gtk_grid_new();
 
     i = 0;
@@ -146,8 +180,11 @@ MergeDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_PERCENT, TRUE, TRUE);
     add_widget_to_table(table, w, _("zoom _X:"), FALSE, i++);
+    g_signal_connect(w, "value-changed", G_CALLBACK(zoom_changed), d);
     d->zoom_x = w;
 
+    d->link = add_toggle_button(table, i++, 1, NGRAPH_LINK_ICON, _("Link"), G_CALLBACK(link_toggled), d);
+
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_PERCENT, TRUE, TRUE);
     add_widget_to_table(table, w, _("zoom _Y:"), FALSE, i++);
     d->zoom_y = w;
@@ -213,7 +250,7 @@ CmMergeOpen(void *w, gpointer client_data)
 {
   struct objlist *obj;
   char *name = NULL;
-  int id, ret, undo;
+  int id, undo;
 
   if (Menulock || Globallock)
     return;
@@ -228,6 +265,7 @@ CmMergeOpen(void *w, gpointer client_data)
   undo = menu_save_undo_single(UNDO_TYPE_CREATE, obj->name);
   id = newobj(obj);
   if (id >= 0) {
+    int ret;
     changefilename(name);
     putobj(obj, "file", id, name);
     MergeDialog(NgraphApp.MergeWin.data.data, id, -1);
@@ -248,8 +286,6 @@ CmMergeClose(void *w, gpointer client_data)
 {
   struct narray farray;
   struct objlist *obj;
-  int i;
-  int num, *array;
 
   if (Menulock || Globallock)
     return;
@@ -257,8 +293,9 @@ CmMergeClose(void *w, gpointer client_data)
     return;
   if (chkobjlastinst(obj) == -1)
     return;
-  SelectDialog(&DlgSelect, obj, _("close merge file (multi select)"), FileCB, (struct narray *) &farray, NULL);
+  SelectDialog(&DlgSelect, obj, _("close merge file (multi select)"), MergeFileCB, (struct narray *) &farray, NULL);
   if (DialogExecute(TopLevel, &DlgSelect) == IDOK) {
+    int i, num, *array;
     num = arraynum(&farray);
     if (num > 0) {
       menu_save_undo_single(UNDO_TYPE_DELETE, obj->name);
@@ -278,8 +315,7 @@ CmMergeUpdate(void *w, gpointer client_data)
 {
   struct narray farray;
   struct objlist *obj;
-  int i, ret, modified;
-  int *array, num;
+  int modified;
 
   if (Menulock || Globallock)
     return;
@@ -287,15 +323,17 @@ CmMergeUpdate(void *w, gpointer client_data)
     return;
   if (chkobjlastinst(obj) == -1)
     return;
-  SelectDialog(&DlgSelect, obj, _("merge file property (multi select)"), FileCB, (struct narray *) &farray, NULL);
+  SelectDialog(&DlgSelect, obj, _("merge file property (multi select)"), MergeFileCB, (struct narray *) &farray, NULL);
   modified = FALSE;
   if (DialogExecute(TopLevel, &DlgSelect) == IDOK) {
+    int i, *array, num;
     num = arraynum(&farray);
     if (num > 0) {
       menu_save_undo_single(UNDO_TYPE_EDIT, obj->name);
     }
     array = arraydata(&farray);
     for (i = 0; i < num; i++) {
+      int ret;
       MergeDialog(NgraphApp.MergeWin.data.data, array[i], -1);
       ret = DialogExecute(TopLevel, &DlgMerge);
       if (ret != IDCANCEL) {
@@ -333,7 +371,7 @@ MergeWinUpdate(struct obj_list_data *d, int clear, int draw)
   case DRAW_REDRAW:
     getobj(Menulocal.obj, "redraw_flag", 0, 0, NULL, &redraw);
     if (redraw) {
-      NgraphApp.Viewer.allclear = TRUE;
+//      NgraphApp.Viewer.allclear = TRUE;
       update_viewer(d);
     } else {
       draw_notify(TRUE);
@@ -361,7 +399,7 @@ merge_list_set_val(struct obj_list_data *d, GtkTreeIter *iter, int row)
 	list_store_set_string(GTK_WIDGET(d->text), iter, i, CHK_STR(bfile));
 	g_free(bfile);
       } else {
-	list_store_set_string(GTK_WIDGET(d->text), iter, i, "....................");
+	list_store_set_string(GTK_WIDGET(d->text), iter, i, FILL_STRING);
       }
       break;
     case MERG_WIN_COL_HIDDEN:
@@ -385,7 +423,7 @@ static void
 popup_show_cb(GtkWidget *widget, gpointer user_data)
 {
   unsigned int i;
-  int sel, num;
+  int sel, num, last_id;
   struct obj_list_data *d;
 
   d = (struct obj_list_data *) user_data;
@@ -394,6 +432,10 @@ popup_show_cb(GtkWidget *widget, gpointer user_data)
   num = chkobjlastinst(d->obj);
   for (i = 1; i < POPUP_ITEM_NUM; i++) {
     switch (i) {
+    case POPUP_ITEM_FOCUS_ALL:
+      last_id = chkobjlastinst(d->obj);
+      gtk_widget_set_sensitive(d->popup_item[i], last_id >= 0);
+      break;
     case POPUP_ITEM_TOP:
     case POPUP_ITEM_UP:
       gtk_widget_set_sensitive(d->popup_item[i], sel > 0 && sel <= num);
@@ -440,7 +482,7 @@ init_dnd(struct SubWin *d)
 }
 
 GtkWidget *
-dreate_merge_list(struct SubWin *d)
+create_merge_list(struct SubWin *d)
 {
   if (d->Win) {
     return d->Win;
