@@ -1021,6 +1021,7 @@ MiscDialogSetupItem(GtkWidget *w, struct MiscDialog *d)
     gtk_source_style_scheme_chooser_set_style_scheme(GTK_SOURCE_STYLE_SCHEME_CHOOSER(d->source_style),
                                                      style);
   }
+  combo_box_set_active(d->decimalsign, Menulocal.default_decimalsign);
 }
 
 static void
@@ -1207,8 +1208,6 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox2), vbox2, TRUE, TRUE, 4);
 
 
-    vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
-
     frame = gtk_frame_new(_("Size"));
     table = gtk_grid_new();
 
@@ -1229,6 +1228,8 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(vbox2), frame, FALSE, FALSE, 4);
 
 
+    vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+
     frame = gtk_frame_new(_("Font"));
     table = gtk_grid_new();
 
@@ -1276,6 +1277,13 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
     add_widget_to_table(table, w, _("_Source style:"), FALSE, i++);
     d->source_style = w;
 
+    w = combo_box_create();
+    add_widget_to_table(table, w, _("_Default decimalsign:"), FALSE, i++);
+    for (j = 0; decimalsign_char[j]; j++) {
+      combo_box_append_text(w, _(decimalsign_char[j]));
+    }
+    d->decimalsign = w;
+
     gtk_container_add(GTK_CONTAINER(frame), table);
     gtk_box_pack_start(GTK_BOX(vbox2), frame, TRUE, TRUE, 4);
 
@@ -1418,6 +1426,8 @@ MiscDialogClose(GtkWidget *w, void *data)
     }
     Menulocal.source_style_id = g_strdup(source_style_id);
   }
+  Menulocal.default_decimalsign = combo_box_get_active(d->decimalsign);
+  gra_set_default_decimalsign(get_gra_decimalsign_type(Menulocal.default_decimalsign));
   if (d->ret == IDSAVE) {
     save_config(SAVE_CONFIG_TYPE_MISC);
   }
@@ -1737,7 +1747,7 @@ CmOptionSaveNgp(void *w, gpointer client_data)
 
   snprintf(mes, sizeof(mes), _("Saving `%.128s'."), ngpfile);
   SetStatusBar(mes);
-  SaveDrawrable(ngpfile, FALSE, FALSE);
+  SaveDrawrable(ngpfile, FALSE, FALSE, FALSE);
   ResetStatusBar();
   g_free(ngpfile);
   return;
