@@ -14,6 +14,7 @@
 #include <glib.h>
 
 #include "nhash.h"
+#include "nstring.h"
 #include "odata.h"
 
 #include "math_scanner.h"
@@ -872,7 +873,7 @@ math_expression_get_string(MathExpression *expression)
     default:
       if (math_expression_calculate(exp, &val) == 0) {
 	if (val.type == MATH_VALUE_NORMAL) {
-	  g_string_append_printf(gstr, "%G", val.val);
+	  n_gstr_append_printf_double(gstr, "%G", val.val);
 	} else {
 	  const char *str;
 	  str = math_special_value_to_string(&val);
@@ -2033,7 +2034,7 @@ val2str(MathValue *val)
 
   tmp = math_special_value_to_string(val);
   if (tmp == NULL) {
-    str = g_strdup_printf("%G", val->val);
+    str = n_strdup_printf_double("%G", val->val);
   } else {
     str = g_strdup(tmp);
   }
