@@ -29,9 +29,17 @@ static struct math_token *st_look_ahead_token = NULL;
 static MathExpression * parse_expression(struct math_string *mstr, MathEquation *eq, int *err);
 static MathExpression * parse_expression_list(struct math_string *mstr, MathEquation *eq, int inside_block, int *err);
 static MathExpression * parse_unary_expression(struct math_string *mstr, MathEquation *eq, int *err);
+static MathExpression * parse_block_expression(struct math_string *str, MathEquation *eq, int *err);
+
+struct parsing_info
+{
+  struct math_string str;
+  struct math_token *st_look_ahead_token;
+  MathExpression *exp;
+};
 
 static struct math_token *
-my_get_token(struct math_string *str)
+my_get_token(struct math_string *str, MathEquation *eq)
 {
   struct math_token *token;
 
@@ -40,7 +48,7 @@ my_get_token(struct math_string *str)
     st_look_ahead_token = token->next;
     token->next = NULL;
   } else {
-    token = math_scanner_get_token(str);
+    token = math_scanner_get_token(str, eq);
   }
 
   return token;
@@ -53,13 +61,87 @@ unget_token(struct math_token *token)
   st_look_ahead_token = token;
 }
 
+static void
+free_token(struct math_token *token)
+{
+  struct math_token *tmp;
+
+  while (token) {
+    tmp = token->next;
+    math_scanner_free_token(token);
+    token = tmp;
+  }
+}
+
+static void
+reset_token(struct math_token *cur)
+{
+  free_token(st_look_ahead_token);
+  st_look_ahead_token = cur;
+}
+
+static struct math_token *
+dup_token(struct math_token *token)
+{
+  struct math_token *new_token;
+  new_token = math_scanner_dup_token(token);
+  if (token->next) {
+    new_token->next = dup_token(token->next);
+  }
+  return new_token;
+}
+
+static struct parsing_info *
+save_parsing_info(struct math_string *str, MathExpression *exp)
+{
+  struct parsing_info *info;
+  info = g_malloc0(sizeof(*info));
+  if (info == NULL) {
+    return NULL;
+  }
+  info->str = *str;
+  if (st_look_ahead_token) {
+    info->st_look_ahead_token = dup_token(st_look_ahead_token);
+    if (info->st_look_ahead_token == NULL) {
+      g_free(info);
+      return NULL;
+    }
+  }
+  info->exp = exp;
+  return info;
+}
+
+static void
+free_parsing_info(struct parsing_info *info)
+{
+  if (info == NULL) {
+    return;
+  }
+  if (info->st_look_ahead_token) {
+    free_token(info->st_look_ahead_token);
+  }
+  g_free(info);
+}
+
 static MathExpression *
-parse_array_expression(struct math_string *str, MathEquation *eq, const char *name, int *err)
+restore_parsing_info(struct parsing_info *info, struct math_string *str)
+{
+  if (info == NULL) {
+    return NULL;
+  }
+  *str = info->str;
+  reset_token(info->st_look_ahead_token);
+  info->st_look_ahead_token = NULL;
+  return info->exp;
+}
+
+static MathExpression *
+parse_array_expression(struct math_string *str, MathEquation *eq, const char *name, int is_string, int *err)
 {
   struct math_token *token;
   MathExpression *operand, *exp;
 
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   if (token == NULL) {
     *err = MATH_ERROR_MEMORY;
     return NULL;
@@ -70,7 +152,7 @@ parse_array_expression(struct math_string *str, MathEquation *eq, const char *na
   if (operand == NULL)
     return NULL;
 
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   if (token == NULL) {
     *err = MATH_ERROR_MEMORY;
     math_expression_free(operand);
@@ -87,7 +169,7 @@ parse_array_expression(struct math_string *str, MathEquation *eq, const char *na
 
   math_scanner_free_token(token);
 
-  exp = math_array_expression_new(eq, name, operand, err);
+  exp = math_array_expression_new(eq, name, operand, is_string, err);
   if (exp == NULL) {
     math_expression_free(operand);
     return NULL;
@@ -103,7 +185,7 @@ parse_primary_expression(struct math_string *str, MathEquation *eq, int *err)
   MathExpression *exp;
   MathValue val;
 
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   if (token == NULL) {
     *err = MATH_ERROR_MEMORY;
     return NULL;
@@ -115,11 +197,40 @@ parse_primary_expression(struct math_string *str, MathEquation *eq, int *err)
     val.type = MATH_VALUE_NORMAL;
     exp = math_double_expression_new(eq, &val, err);
     break;
-  case MATH_TOKEN_TYPE_SYMBOL:
-    token2 = my_get_token(str);
+  case MATH_TOKEN_TYPE_DOUBLE_QUOTED_STRING:
+    exp = math_string_expression_new(eq, token->data.str->str, TRUE, err);
+    if (exp == NULL) {
+      math_equation_set_parse_error(eq, token->ptr, str);
+      math_scanner_free_token(token);
+      return NULL;
+    }
+    break;
+  case MATH_TOKEN_TYPE_SINGLE_QUOTED_STRING:
+    exp = math_string_expression_new(eq, token->data.str->str, FALSE, err);
+    break;
+  case MATH_TOKEN_TYPE_UNTERMINATED_STRING:
+    *err = MATH_ERROR_UNTERMINATED_STRING;
+    math_equation_set_parse_error(eq, token->ptr, str);
+    math_scanner_free_token(token);
+    return NULL;
+  case MATH_TOKEN_TYPE_STRING_VARIABLE:
+    token2 = my_get_token(str, eq);
     unget_token(token2);
     if (token2->type == MATH_TOKEN_TYPE_LB) {
-      exp = parse_array_expression(str, eq, token->data.sym, err);
+      exp = parse_array_expression(str, eq, token->data.sym, TRUE, err);
+      if (exp == NULL) {
+	math_scanner_free_token(token);
+	return NULL;
+      }
+    } else {
+      exp = math_string_variable_expression_new(eq, token->data.sym, err);
+    }
+    break;
+  case MATH_TOKEN_TYPE_SYMBOL:
+    token2 = my_get_token(str, eq);
+    unget_token(token2);
+    if (token2->type == MATH_TOKEN_TYPE_LB) {
+      exp = parse_array_expression(str, eq, token->data.sym, FALSE, err);
       if (exp == NULL) {
 	math_scanner_free_token(token);
 	return NULL;
@@ -156,7 +267,7 @@ parse_primary_expression(struct math_string *str, MathEquation *eq, int *err)
     if (exp == NULL)
       return NULL;
 
-    token = my_get_token(str);
+    token = my_get_token(str, eq);
     if (token == NULL) {
       *err = MATH_ERROR_MEMORY;
       math_expression_free(exp);
@@ -171,6 +282,10 @@ parse_primary_expression(struct math_string *str, MathEquation *eq, int *err)
       return NULL;
     }
     break;
+  case MATH_TOKEN_TYPE_LC:
+    exp = parse_block_expression(str, eq, err);
+    break;
+  case MATH_TOKEN_TYPE_EOEQ_ASSIGN:
   case MATH_TOKEN_TYPE_EOEQ:
     *err = MATH_ERROR_EOEQ;
     math_scanner_free_token(token);
@@ -200,22 +315,57 @@ free_arg_list(MathExpression **argv)
 static MathExpression *
 get_argument(struct math_string *str, MathEquation *eq, struct math_function_parameter *fprm, int i, int *err)
 {
-  struct math_token *token;
   MathExpression *exp;
   int argc;
 
   argc = math_function_get_arg_type_num(fprm);
-  if (fprm->arg_type && i < argc && fprm->arg_type[i] == MATH_FUNCTION_ARG_TYPE_ARRAY) {
-    token = my_get_token(str);
-    if (token->type != MATH_TOKEN_TYPE_SYMBOL) {
-      *err = MATH_ERROR_INVALID_FDEF;
-      math_equation_set_parse_error(eq, token->ptr, str);
+  if (fprm->arg_type && i < argc) {
+    struct math_token *token;
+    switch (fprm->arg_type[i]) {
+    case MATH_FUNCTION_ARG_TYPE_ARRAY:
+      token = my_get_token(str, eq);
+      if (token->type != MATH_TOKEN_TYPE_SYMBOL) {
+	*err = MATH_ERROR_INVALID_ARG;
+	math_equation_set_parse_error(eq, token->ptr, str);
+	math_scanner_free_token(token);
+	/* invalid argument */
+	return NULL;
+      }
+      exp = math_array_argument_expression_new(eq, token->data.sym, err);
       math_scanner_free_token(token);
-      /* invalid argument */
-      return NULL;
+      break;
+    case MATH_FUNCTION_ARG_TYPE_STRING_ARRAY:
+      token = my_get_token(str, eq);
+      if (token->type != MATH_TOKEN_TYPE_STRING_VARIABLE) {
+	*err = MATH_ERROR_INVALID_ARG;
+	math_equation_set_parse_error(eq, token->ptr, str);
+	math_scanner_free_token(token);
+	/* invalid argument */
+	return NULL;
+      }
+      exp = math_string_array_argument_expression_new(eq, token->data.sym, err);
+      math_scanner_free_token(token);
+      break;
+    case MATH_FUNCTION_ARG_TYPE_ARRAY_COMMON:
+      token = my_get_token(str, eq);
+      if (token->type == MATH_TOKEN_TYPE_SYMBOL) {
+        exp = math_array_argument_expression_new(eq, token->data.sym, err);
+        math_scanner_free_token(token);
+      } else if (token->type == MATH_TOKEN_TYPE_STRING_VARIABLE) {
+        exp = math_string_array_argument_expression_new(eq, token->data.sym, err);
+        math_scanner_free_token(token);
+      } else {
+	*err = MATH_ERROR_INVALID_ARG;
+	math_equation_set_parse_error(eq, token->ptr, str);
+	math_scanner_free_token(token);
+        return NULL;
+	/* invalid argument */
+      }
+      break;
+    default:
+      exp = parse_expression(str, eq, err);
+      break;
     }
-    exp = math_array_argument_expression_new(eq, token->data.sym, err);
-    math_scanner_free_token(token);
   } else {
     exp = parse_expression(str, eq, err);
   }
@@ -232,7 +382,7 @@ parse_argument_list(struct math_string *str, MathEquation *eq, struct math_funct
 
   argv = *buf;
 
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   unget_token(token);
   if (token->type == MATH_TOKEN_TYPE_RP) {
     return 0;
@@ -247,7 +397,7 @@ parse_argument_list(struct math_string *str, MathEquation *eq, struct math_funct
   argv[0] = exp;
   argv[1] = NULL;
   for (i = 1; ; i++) {
-    token = my_get_token(str);
+    token = my_get_token(str, eq);
     if (token == NULL) {
       *err = MATH_ERROR_MEMORY;
       free_arg_list(argv);
@@ -296,7 +446,7 @@ static MathExpression *
 create_math_func(struct math_string *str, MathEquation *eq, struct math_token *name, int *err)
 {
   struct math_function_parameter *fprm;
-  int i, arg_max, argc, pos_id;
+  int arg_max, argc, pos_id;
   MathExpression **argv, *exp;
   struct math_token *token;
 
@@ -321,7 +471,7 @@ create_math_func(struct math_string *str, MathEquation *eq, struct math_token *n
     return NULL;
   }
 
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   if (token->type != MATH_TOKEN_TYPE_RP) {
     *err = MATH_ERROR_MISS_RP;
     math_equation_set_parse_error(eq, token->ptr, str);
@@ -330,12 +480,13 @@ create_math_func(struct math_string *str, MathEquation *eq, struct math_token *n
     math_scanner_free_token(token);
     return NULL;
   }
-  math_scanner_free_token(token);
 
   if (argc < fprm->argc) {
+    int i;
     for (i = argc; i < fprm->argc; i++) {
       argv[i] = math_double_expression_new(eq, &MATH_VALUE_ZERO, err);
       if (argv[i] == NULL) {
+        math_scanner_free_token(token);
 	free_arg_list(argv);
 	g_free(argv);
 	return NULL;
@@ -344,7 +495,9 @@ create_math_func(struct math_string *str, MathEquation *eq, struct math_token *n
     argc = fprm->argc;
   } else if (fprm->argc >= 0 && argc > fprm->argc) {
     *err = MATH_ERROR_ARG_NUM;
+    math_equation_set_parse_error(eq, token->ptr, str);
     math_equation_set_func_arg_num_error(eq, fprm, argc);
+    math_scanner_free_token(token);
     free_arg_list(argv);
     g_free(argv);
     return NULL;
@@ -353,7 +506,19 @@ create_math_func(struct math_string *str, MathEquation *eq, struct math_token *n
   pos_id = math_equation_add_pos_func(eq, fprm);
   if (pos_id == MATH_ERROR_INVALID_FUNC) {
     *err = MATH_ERROR_INVALID_FUNC;
+    math_equation_set_parse_error(eq, token->ptr, str);
     math_equation_set_func_error(eq, fprm);
+    math_scanner_free_token(token);
+    free_arg_list(argv);
+    g_free(argv);
+    return NULL;
+  }
+
+  if (fprm->type == MATH_FUNCTION_TYPE_CALLBACK && eq->func_def) {
+    *err = MATH_ERROR_INVALID_FUNC;
+    math_equation_set_parse_error(eq, token->ptr, str);
+    math_equation_set_func_error(eq, fprm);
+    math_scanner_free_token(token);
     free_arg_list(argv);
     g_free(argv);
     return NULL;
@@ -361,10 +526,15 @@ create_math_func(struct math_string *str, MathEquation *eq, struct math_token *n
 
   exp = math_func_call_expression_new(eq, fprm, argc, argv, pos_id, err);
   if (exp == NULL) {
+    if (*err == MATH_ERROR_INVALID_ARG) {
+      math_equation_set_parse_error(eq, token->ptr, str);
+    }
+    math_scanner_free_token(token);
     free_arg_list(argv);
     g_free(argv);
     return NULL;
   }
+  math_scanner_free_token(token);
 
   return exp;
 }
@@ -375,14 +545,14 @@ parse_func_expression(struct math_string *str, MathEquation *eq, int *err)
   struct math_token *token1, *token2;
   MathExpression *exp;
 
-  token1 = my_get_token(str);
+  token1 = my_get_token(str, eq);
   if (token1 == NULL) {
     return NULL;
   }
 
   switch (token1->type) {
   case MATH_TOKEN_TYPE_SYMBOL:
-    token2 = my_get_token(str);
+    token2 = my_get_token(str, eq);
     if (token2 == NULL) {
       math_scanner_free_token(token1);
       return NULL;
@@ -420,7 +590,7 @@ parse_factorial_expression(struct math_string *str, MathEquation *eq, int *err)
   }
 
   for (;;) {
-    token = my_get_token(str);
+    token = my_get_token(str, eq);
     if (token == NULL) {
       *err = MATH_ERROR_MEMORY;
       math_expression_free(exp);
@@ -465,7 +635,7 @@ parse_power_expression(struct math_string *str, MathEquation *eq, int *err)
     return NULL;
   }
 
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   if (token == NULL) {
     *err = MATH_ERROR_MEMORY;
     math_expression_free(exp);
@@ -511,7 +681,7 @@ parse_unary_expression(struct math_string *str, MathEquation *eq, int *err)
   struct math_token *token;
   MathExpression *exp, *operand;
 
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   if (token == NULL) {
     *err = MATH_ERROR_MEMORY;
     return NULL;
@@ -536,6 +706,19 @@ parse_unary_expression(struct math_string *str, MathEquation *eq, int *err)
 	return NULL;
       }
       break;
+    case MATH_OPERATOR_TYPE_FACT:
+      operand = parse_unary_expression(str, eq, err);
+      if (operand == NULL) {
+	math_scanner_free_token(token);
+	return NULL;
+      }
+      exp = math_unary_expression_new(MATH_EXPRESSION_TYPE_NOT, eq, operand, err);
+      if (exp == NULL) {
+	math_scanner_free_token(token);
+	math_expression_free(operand);
+	return NULL;
+      }
+      break;
     default:
       *err = MATH_ERROR_UNEXP_OPE;
       math_equation_set_parse_error(eq, token->ptr, str);
@@ -553,10 +736,85 @@ parse_unary_expression(struct math_string *str, MathEquation *eq, int *err)
 }
 
 static MathExpression *
-CREATE_PARSER3_FUNC(multiplicative, unary,
-		    MATH_OPERATOR_TYPE_MUL, MATH_EXPRESSION_TYPE_MUL,
-		    MATH_OPERATOR_TYPE_DIV, MATH_EXPRESSION_TYPE_DIV,
-		    MATH_OPERATOR_TYPE_MOD, MATH_EXPRESSION_TYPE_MOD);
+parse_multiplicative_expression(struct math_string *str, MathEquation *eq, int *err)
+{
+  struct math_token *token;
+  MathExpression *left, *right, *exp;
+  enum MATH_EXPRESSION_TYPE type;
+
+  exp = parse_unary_expression(str, eq, err);
+  if (exp == NULL) {
+    return NULL;
+  }
+  for (;;) {
+    token = my_get_token(str, eq);
+    if (token == NULL) {
+      *err = MATH_ERROR_MEMORY;
+      math_expression_free(exp);
+      return NULL;
+    }
+    switch (token->type) {
+    case MATH_TOKEN_TYPE_OPERATOR:
+      switch (token->data.op) {
+      case MATH_OPERATOR_TYPE_MUL:
+      case MATH_OPERATOR_TYPE_DIV:
+      case MATH_OPERATOR_TYPE_MOD:
+	right = parse_unary_expression(str, eq, err);
+	if (right == NULL) {
+	  math_expression_free(exp);
+	  math_scanner_free_token(token);
+	  return NULL;
+	}
+	if (token->data.op == MATH_OPERATOR_TYPE_MUL) {
+	  type = MATH_EXPRESSION_TYPE_MUL;
+	} else if (token->data.op == MATH_OPERATOR_TYPE_DIV) {
+	  type = MATH_EXPRESSION_TYPE_DIV;
+	} else if (token->data.op == MATH_OPERATOR_TYPE_MOD) {
+	  type = MATH_EXPRESSION_TYPE_MOD;
+	} else {
+	  type = 0;
+	}
+	left = exp;
+	math_scanner_free_token(token);
+	exp = math_binary_expression_new(type, eq, left, right, err);
+	if (exp == NULL) {
+	  math_expression_free(left);
+	  math_expression_free(right);
+	  return NULL;
+	}
+	break;
+      default:
+	unget_token(token);
+	goto End;
+      }
+      break;
+    case MATH_TOKEN_TYPE_CONST:
+    case MATH_TOKEN_TYPE_NUMERIC:
+    case MATH_TOKEN_TYPE_SYMBOL:
+    case MATH_TOKEN_TYPE_LP:
+    case MATH_TOKEN_TYPE_LC:
+      unget_token(token);
+      right = parse_unary_expression(str, eq, err);
+      if (right == NULL) {
+	math_expression_free(exp);
+	return NULL;
+      }
+      left = exp;
+      exp = math_binary_expression_new(MATH_EXPRESSION_TYPE_MUL, eq, left, right, err);
+      if (exp == NULL) {
+	math_expression_free(left);
+	math_expression_free(right);
+	return NULL;
+      }
+      break;
+    default:
+      unget_token(token);
+      goto End;
+    }
+  }
+ End:
+  return exp;
+}
 
 static MathExpression *
 CREATE_PARSER2_FUNC(additive, multiplicative,
@@ -667,7 +925,6 @@ parse_assign_expression(struct math_string *str, MathEquation *eq, enum MATH_OPE
 
   rexp = parse_expression(str, eq, err);
   if (rexp == NULL) {
-    math_expression_free(lexp);
     return NULL;
   }
 
@@ -676,7 +933,6 @@ parse_assign_expression(struct math_string *str, MathEquation *eq, enum MATH_OPE
   } else {
     exp = math_assign_expression_new(MATH_EXPRESSION_TYPE_ASSIGN, eq, lexp, rexp, op, err);
     if (exp == NULL) {
-      math_expression_free(lexp);
       math_expression_free(rexp);
     }
   }
@@ -685,12 +941,14 @@ parse_assign_expression(struct math_string *str, MathEquation *eq, enum MATH_OPE
 }
 
 static int
-parse_parameter_list(struct math_string *str, MathEquation *eq, MathExpression *func, int *err)
+parse_parameter_list(struct math_string *str, MathExpression *func, int *err)
 {
   struct math_token *token;
   enum MATH_FUNCTION_ARG_TYPE type;
+  MathEquation *eq;
 
-  token = my_get_token(str);
+  eq = func->equation;
+  token = my_get_token(str, eq);
   if (token == NULL) {
     *err = MATH_ERROR_MEMORY;
     return 1;
@@ -702,28 +960,39 @@ parse_parameter_list(struct math_string *str, MathEquation *eq, MathExpression *
   }
   math_scanner_free_token(token);
 
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   if (token->type == MATH_TOKEN_TYPE_RP) {
     return 0;
   }
 
   unget_token(token);
   for (;;) {
-    token = my_get_token(str);
+    token = my_get_token(str, eq);
     if (token == NULL) {
       *err = MATH_ERROR_MEMORY;
       return 1;
     }
 
-    if (token->type == MATH_TOKEN_TYPE_ARRAY_PREFIX) {
-      type = MATH_FUNCTION_ARG_TYPE_ARRAY;
+    switch (token->type) {
+    case MATH_TOKEN_TYPE_ARRAY_PREFIX:
       math_scanner_free_token(token);
-      token = my_get_token(str);
-    } else {
+      token = my_get_token(str, eq);
+      if (token->type == MATH_TOKEN_TYPE_SYMBOL) {
+	type = MATH_FUNCTION_ARG_TYPE_ARRAY;
+      } else {
+	type = MATH_FUNCTION_ARG_TYPE_STRING_ARRAY;
+      }
+      break;
+    case MATH_TOKEN_TYPE_STRING_VARIABLE:
+      type = MATH_FUNCTION_ARG_TYPE_STRING;
+      break;
+    default:
       type = MATH_FUNCTION_ARG_TYPE_DOUBLE;
+      break;
     }
 
-    if (token->type != MATH_TOKEN_TYPE_SYMBOL) {
+    if (token->type != MATH_TOKEN_TYPE_SYMBOL &&
+	token->type != MATH_TOKEN_TYPE_STRING_VARIABLE) {
       math_scanner_free_token(token);
       return 1;
     }
@@ -734,7 +1003,7 @@ parse_parameter_list(struct math_string *str, MathEquation *eq, MathExpression *
     }
     math_scanner_free_token(token);
 
-    token = my_get_token(str);
+    token = my_get_token(str, eq);
     if (token == NULL) {
       *err = MATH_ERROR_MEMORY;
       return 1;
@@ -747,7 +1016,7 @@ parse_parameter_list(struct math_string *str, MathEquation *eq, MathExpression *
     math_scanner_free_token(token);
   }
 
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   if (token == NULL) {
     *err = MATH_ERROR_MEMORY;
     return 1;
@@ -766,23 +1035,15 @@ static MathExpression *
 parse_block_expression(struct math_string *str, MathEquation *eq, int *err)
 {
   struct math_token *token;
-  MathExpression *exp;
+  MathExpression *exp, *block;
 
-  token = my_get_token(str);
-  if (token == NULL) {
-    return NULL;
-  }
-
-  if (token->type != MATH_TOKEN_TYPE_LC) {
-    math_scanner_free_token(token);
-    return NULL;
-  }
-  math_scanner_free_token(token);
-
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   if (token->type == MATH_TOKEN_TYPE_RC) {
+    MathValue val;
     math_scanner_free_token(token);
-    return NULL;
+    val.val = 0;
+    val.type = MATH_VALUE_NORMAL;
+    return math_double_expression_new(eq, &val, err);
   }
   unget_token(token);
 
@@ -790,14 +1051,23 @@ parse_block_expression(struct math_string *str, MathEquation *eq, int *err)
   if (exp == NULL)
     return NULL;
 
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   if (token->type != MATH_TOKEN_TYPE_RC) {
+    *err = MATH_ERROR_MISS_RC;
+    math_equation_set_parse_error(eq, token->ptr, str);
     math_scanner_free_token(token);
+    math_expression_free(exp);
     return NULL;
   }
   math_scanner_free_token(token);
 
-  return exp;
+  block = math_expression_new(MATH_EXPRESSION_TYPE_BLOCK, eq, err);
+  if (block == NULL) {
+    math_expression_free(exp);
+    return NULL;
+  }
+  block->u.exp = exp;
+  return block;
 }
 
 static void
@@ -826,9 +1096,10 @@ parse_func_def_expression(struct math_string *str, MathEquation *eq, int *err)
 {
   struct math_token *fname;
   MathExpression *exp, *block;
+  struct math_token *token;
 
   /* get name of the function */
-  fname = my_get_token(str);
+  fname = my_get_token(str, eq);
   if (fname == NULL) {
     return NULL;
   }
@@ -845,8 +1116,8 @@ parse_func_def_expression(struct math_string *str, MathEquation *eq, int *err)
   }
 
   /* get parameters */
-  if (parse_parameter_list(str, eq, exp, err)) {
-    math_equation_finish_user_func_definition(eq, NULL, NULL);
+  if (parse_parameter_list(str, exp, err)) {
+    math_equation_finish_user_func_definition(eq, NULL, NULL, NULL, NULL);
     free_func_prm(exp->u.func.fprm);
     math_scanner_free_token(fname);
     math_expression_free(exp);
@@ -854,16 +1125,25 @@ parse_func_def_expression(struct math_string *str, MathEquation *eq, int *err)
   }
 
   if (math_function_expression_register_arg(exp)) {
-    math_equation_finish_user_func_definition(eq, NULL, NULL);
+    math_equation_finish_user_func_definition(eq, NULL, NULL, NULL, NULL);
     free_func_prm(exp->u.func.fprm);
     math_scanner_free_token(fname);
     math_expression_free(exp);
   }
 
   /* get block */
+  token = my_get_token(str, eq);
+  if (token == NULL) {
+    return NULL;
+  }
+  if (token->type != MATH_TOKEN_TYPE_LC) {
+    math_scanner_free_token(token);
+    return NULL;
+  }
+  math_scanner_free_token(token);
   block = parse_block_expression(str, eq, err);
   if (block == NULL) {
-    math_equation_finish_user_func_definition(eq, NULL, NULL);
+    math_equation_finish_user_func_definition(eq, NULL, NULL, NULL, NULL);
     free_func_prm(exp->u.func.fprm);
     math_scanner_free_token(fname);
     math_expression_free(exp);
@@ -871,7 +1151,7 @@ parse_func_def_expression(struct math_string *str, MathEquation *eq, int *err)
   }
 
   if (math_function_expression_set_function(eq, exp, fname->data.sym, block)) {
-    math_equation_finish_user_func_definition(eq, NULL, NULL);
+    math_equation_finish_user_func_definition(eq, NULL, NULL, NULL, NULL);
     free_func_prm(exp->u.func.fprm);
     math_scanner_free_token(fname);
     math_expression_free(exp);
@@ -895,7 +1175,7 @@ parse_const_def_expression(struct math_string *str, MathEquation *eq, int *err)
   }
 
   /* get name of the constant */
-  cname = my_get_token(str);
+  cname = my_get_token(str, eq);
   if (cname == NULL) {
     return NULL;
   }
@@ -907,14 +1187,16 @@ parse_const_def_expression(struct math_string *str, MathEquation *eq, int *err)
     return NULL;
   }
 
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   if (token == NULL) {
     *err = MATH_ERROR_MEMORY;
     math_scanner_free_token(cname);
     return NULL;
   }
 
-  if (token->type != MATH_TOKEN_TYPE_OPERATOR || token->data.op != MATH_OPERATOR_TYPE_ASSIGN) {
+  if ((token->type != MATH_TOKEN_TYPE_OPERATOR &&
+       token->type != MATH_TOKEN_TYPE_EOEQ_ASSIGN) ||
+      token->data.op != MATH_OPERATOR_TYPE_ASSIGN) {
     if (token->type == MATH_TOKEN_TYPE_EOEQ) {
       *err = MATH_ERROR_EOEQ;
     } else {
@@ -925,24 +1207,53 @@ parse_const_def_expression(struct math_string *str, MathEquation *eq, int *err)
     math_scanner_free_token(cname);
     return NULL;
   }
-  math_scanner_free_token(token);
 
   eq->func_def = TRUE;
   exp = parse_expression(str, eq, err);
   eq->func_def = FALSE;
   if (exp == NULL) {
     math_scanner_free_token(cname);
+    math_scanner_free_token(token);
     return NULL;
   }
 
   cdef = math_constant_definition_expression_new(eq, cname->data.sym, exp, err);
   math_scanner_free_token(cname);
   if (cdef == NULL) {
+    math_equation_set_parse_error(eq, token->ptr, str);
     math_expression_free(exp);
+    math_scanner_free_token(token);
+    return NULL;
+  }
+  math_scanner_free_token(token);
+
+  return cdef;
+}
+
+static MathExpression *
+parse_string_assign_expression(struct math_string *str, MathEquation *eq, struct math_token *token, MathExpression *lexp, int *err)
+{
+  MathExpression *exp, *rexp;
+
+  if ((token->type != MATH_TOKEN_TYPE_OPERATOR &&
+       token->type != MATH_TOKEN_TYPE_EOEQ_ASSIGN) ||
+      token->data.op != MATH_OPERATOR_TYPE_ASSIGN) {
+    *err = MATH_ERROR_UNEXP_OPE;
+    math_equation_set_parse_error(eq, token->ptr, str);
     return NULL;
   }
 
-  return cdef;
+  rexp = parse_expression(str, eq, err);
+  if (rexp == NULL) {
+    return NULL;
+  }
+
+  exp = math_assign_expression_new(MATH_EXPRESSION_TYPE_STRING_ASSIGN, eq, lexp, rexp, token->data.op, err);
+  if (exp == NULL) {
+    math_expression_free(rexp);
+    return NULL;
+  }
+  return exp;
 }
 
 static MathExpression *
@@ -950,6 +1261,7 @@ parse_expression(struct math_string *str, MathEquation *eq, int *err)
 {
   struct math_token *token;
   MathExpression *exp;
+  struct parsing_info *info;
 
   exp = parse_or_expression(str, eq, err);
   if (exp == NULL)
@@ -957,13 +1269,15 @@ parse_expression(struct math_string *str, MathEquation *eq, int *err)
 
   switch (exp->type) {
   case MATH_EXPRESSION_TYPE_VARIABLE:
+  case MATH_EXPRESSION_TYPE_STRING_VARIABLE:
   case MATH_EXPRESSION_TYPE_ARRAY:
+  case MATH_EXPRESSION_TYPE_STRING_ARRAY:
     break;
   default:
     goto End;
   }
 
-  token = my_get_token(str);
+  token = my_get_token(str, eq);
   if (token == NULL) {
     *err = MATH_ERROR_MEMORY;
     math_expression_free(exp);
@@ -971,17 +1285,51 @@ parse_expression(struct math_string *str, MathEquation *eq, int *err)
   }
 
   switch (token->type) {
+  case MATH_TOKEN_TYPE_EOEQ_ASSIGN:
   case MATH_TOKEN_TYPE_OPERATOR:
     switch (token->data.op) {
-    case MATH_OPERATOR_TYPE_ASSIGN:
     case MATH_OPERATOR_TYPE_POW_ASSIGN:
     case MATH_OPERATOR_TYPE_MOD_ASSIGN:
     case MATH_OPERATOR_TYPE_DIV_ASSIGN:
     case MATH_OPERATOR_TYPE_MUL_ASSIGN:
     case MATH_OPERATOR_TYPE_PLUS_ASSIGN:
     case MATH_OPERATOR_TYPE_MINUS_ASSIGN:
-      exp = parse_assign_expression(str, eq, token->data.op, exp, err);
-      math_scanner_free_token(token);
+      if (exp->type == MATH_EXPRESSION_TYPE_STRING_VARIABLE ||
+	  exp->type == MATH_EXPRESSION_TYPE_STRING_ARRAY) {
+	*err = MATH_ERROR_UNEXP_OPE;
+	math_equation_set_parse_error(eq, token->ptr, str);
+	math_scanner_free_token(token);
+	math_expression_free(exp);
+	return NULL;
+      }
+      /* fall through */
+    case MATH_OPERATOR_TYPE_ASSIGN:
+      info = save_parsing_info(str, exp);
+      if (info == NULL) {
+	*err = MATH_ERROR_MEMORY;
+	math_scanner_free_token(token);
+	math_expression_free(exp);
+	return NULL;
+      }
+      if (exp->type == MATH_EXPRESSION_TYPE_STRING_VARIABLE ||
+	  exp->type == MATH_EXPRESSION_TYPE_STRING_ARRAY) {
+	exp = parse_string_assign_expression(str, eq, token, exp, err);
+      } else {
+	exp = parse_assign_expression(str, eq, token->data.op, exp, err);
+      }
+      if (exp == NULL) {
+	if (token->type == MATH_TOKEN_TYPE_EOEQ_ASSIGN && *err != MATH_ERROR_NONE) {
+	  *err = MATH_ERROR_NONE;
+	  exp = restore_parsing_info(info, str);
+	  unget_token(token);
+	} else {
+	  math_expression_free(info->exp);
+	  math_scanner_free_token(token);
+	}
+      } else {
+	math_scanner_free_token(token);
+      }
+      free_parsing_info(info);
       break;
     default:
       *err = MATH_ERROR_UNEXP_OPE;
@@ -1019,7 +1367,7 @@ parse_expression_list(struct math_string *str, MathEquation *eq, int inside_bloc
 
   top = prev = NULL;
   for (;;) {
-    token = my_get_token(str);
+    token = my_get_token(str, eq);
     if (token == NULL) {
       *err = MATH_ERROR_MEMORY;
       math_expression_free(top);
@@ -1057,6 +1405,9 @@ parse_expression_list(struct math_string *str, MathEquation *eq, int inside_bloc
 	return NULL;
       }
       continue;
+    case MATH_TOKEN_TYPE_EOEQ_ASSIGN:
+      eq->use_eoeq_assign = TRUE;
+      /* fall-through */
     case MATH_TOKEN_TYPE_EOEQ:
       if (str->cur[0] == '\0') {
 	if (inside_block) {
@@ -1101,14 +1452,15 @@ parse_expression_list(struct math_string *str, MathEquation *eq, int inside_bloc
 
     prev = exp;
 
-    token = my_get_token(str);
+    token = my_get_token(str, eq);
     if (token == NULL) {
       *err = MATH_ERROR_MEMORY;
       math_expression_free(top);
       return NULL;
     }
 
-    if (token->type != MATH_TOKEN_TYPE_EOEQ) {
+    if (token->type != MATH_TOKEN_TYPE_EOEQ &&
+	token->type != MATH_TOKEN_TYPE_EOEQ_ASSIGN) {
       if (token->type == MATH_TOKEN_TYPE_RC && inside_block) {
 	unget_token(token);
 	goto End;
@@ -1129,7 +1481,6 @@ parse_expression_list(struct math_string *str, MathEquation *eq, int inside_bloc
 MathExpression *
 math_parser_parse(const char *line, MathEquation *eq, int *err)
 {
-  struct math_token *token, *tmp;
   MathExpression *exp;
   struct math_string str;
 
@@ -1139,13 +1490,7 @@ math_parser_parse(const char *line, MathEquation *eq, int *err)
   st_look_ahead_token = NULL;
   exp = parse_expression_list(&str, eq, 0, err);
 
-  token = st_look_ahead_token;
-  while (token) {
-    tmp = token->next;
-    math_scanner_free_token(token);
-    token = tmp;
-  }
-  st_look_ahead_token = NULL;
+  reset_token(NULL);
 
   return exp;
 }
