@@ -91,6 +91,9 @@ static char *fittypechar[]={
   NULL
 };
 
+#define PRECISION_DISP 7
+#define PRECISION_SAVE 15
+
 struct fitlocal {
   int id, oid;
   MathEquation *codedf[10];
@@ -346,45 +349,44 @@ display_equation(const char *equation)
   ndisplaydialog("\n");
 }
 
-static int
-show_poly_equation(struct fitlocal *fitlocal, enum FIT_OBJ_TYPE type, vector coe)
+static char *
+get_poly_equation(struct fitlocal *fitlocal, enum FIT_OBJ_TYPE type, vector coe, int disp)
 {
   GString *equation;
-  int i;
+  int i, precision;
 
   equation = g_string_new("");
   if (equation == NULL) {
-    return 1;
+    return NULL;
   }
 
+  precision = (disp) ? PRECISION_DISP : PRECISION_SAVE;
   switch (type) {
   case FIT_TYPE_POLY:
     for (i = fitlocal->dim - 1; i > 0; i--) {
       g_string_append_printf(equation,
-			     (i == fitlocal->dim - 1) ? "%.7e*X" : "%+.7e*X",
-			     coe[i]);
+			     (i == fitlocal->dim - 1) ? "%.*gx" : "%+.*gx",
+			     precision, coe[i]);
       if (i > 1) {
 	g_string_append_printf(equation, "^%d", i);
       }
     }
-    g_string_append_printf(equation, "%+.7e", coe[0]);
+    g_string_append_printf(equation, "%+.*g", precision, coe[0]);
     break;
   case FIT_TYPE_POW:
-    g_string_printf(equation, "exp(%.7e)*X^%.7e", coe[0], coe[1]);
+    g_string_printf(equation, "exp(%.*g)x^%.*g", precision, coe[0], precision, coe[1]);
     break;
   case FIT_TYPE_EXP:
-    g_string_printf(equation, "exp(%.7e*X%+.7e)", coe[1], coe[0]);
+    g_string_printf(equation, "exp(%.*gx%+.*g)", precision, coe[1], precision, coe[0]);
     break;
   case  FIT_TYPE_LOG:
-    g_string_printf(equation, "%.7e*Ln(X)%+.7e", coe[1], coe[0]);
+    g_string_printf(equation, "%.*gLn(x)%+.*g", precision, coe[1], precision, coe[0]);
     break;
   case FIT_TYPE_USER:
     /* never reached */
     break;
   }
-  fitlocal->equation = g_string_free(equation, FALSE);
-
-  return 0;
+  return g_string_free(equation, FALSE);
 }
 
 static int
@@ -392,6 +394,7 @@ show_poly_result(struct fitlocal *fitlocal, enum FIT_OBJ_TYPE type, vector coe)
 {
   int j;
   GString *info;
+  char *eqn;
 
   info = g_string_new("");
   if (info == NULL) {
@@ -401,16 +404,16 @@ show_poly_result(struct fitlocal *fitlocal, enum FIT_OBJ_TYPE type, vector coe)
   g_string_append_printf(info, "--------\nfit:%d (^%d)\n", fitlocal->id, fitlocal->oid);
   switch (type) {
   case FIT_TYPE_POLY:
-    g_string_append_printf(info,"Eq: %%0i*X^i (i=0-%d)\n\n", fitlocal->dim - 1);
+    g_string_append_printf(info,"Eq: %%0i*x^i (i=0-%d)\n\n", fitlocal->dim - 1);
     break;
   case FIT_TYPE_POW:
-    g_string_append(info,"Eq: exp(%00)*X^%01\n\n");
+    g_string_append(info,"Eq: exp(%00)*x^%01\n\n");
     break;
   case FIT_TYPE_EXP:
-    g_string_append(info,"Eq: exp(%01*X+%00)\n\n");
+    g_string_append(info,"Eq: exp(%01*x+%00)\n\n");
     break;
   case FIT_TYPE_LOG:
-    g_string_append(info,"Eq: %01*Ln(X)+%00\n\n");
+    g_string_append(info,"Eq: %01*Ln(x)+%00\n\n");
     break;
   case FIT_TYPE_USER:
     /* never reached */
@@ -430,7 +433,12 @@ show_poly_result(struct fitlocal *fitlocal, enum FIT_OBJ_TYPE type, vector coe)
   ndisplaydialog(info->str);
   g_string_free(info, TRUE);
 
-  display_equation(fitlocal->equation);
+  eqn = get_poly_equation(fitlocal, type, coe, TRUE);
+  if (eqn == NULL) {
+    return 1;
+  }
+  display_equation(eqn);
+  g_free(eqn);
 
   return 0;
 }
@@ -452,6 +460,7 @@ fitpoly(struct fitlocal *fitlocal,
   double yy,derror,sy,correlation,wt,sum;
   vector b,x1,x2,coe;
   matrix m;
+  char *eqn;
 
   if (type == FIT_TYPE_POLY) dim=dimension+1;
   else dim=2;
@@ -516,9 +525,11 @@ fitpoly(struct fitlocal *fitlocal,
   fitlocal->correlation=correlation;
   fitlocal->num=num;
 
-  if (show_poly_equation(fitlocal, type, coe)) {
+  eqn = get_poly_equation(fitlocal, type, coe, FALSE);
+  if (eqn == NULL) {
     return FitError_Fatal;
   }
+  fitlocal->equation = eqn;
 
   if (disp && show_poly_result(fitlocal, type, coe)) {
     return FitError_Fatal;
@@ -558,17 +569,18 @@ show_user_result(struct fitlocal *fitlocal, const char *func, int dim, int *tbl,
   return 0;
 }
 
-static int
-show_user_equation(struct fitlocal *fitlocal, const char *func, MathValue *par, int disp)
+static char *
+get_user_equation(const char *func, MathValue *par, int disp)
 {
-  int i, prev_char;
+  int i, prev_char, precision;
   GString *equation;
 
   equation = g_string_sized_new(256);
   if (equation == NULL) {
-    return 1;
+    return NULL;
   }
 
+  precision = (disp) ? PRECISION_DISP : PRECISION_SAVE;
   prev_char = '\0';
   for (i = 0; func[i] != '\0'; i++) {
     double val;
@@ -595,13 +607,13 @@ show_user_equation(struct fitlocal *fitlocal, const char *func, MathValue *par,
 	/* fall-through */
       case '+':
 	g_string_truncate(equation, equation->len - 1);
-	format = "%+.7e";
+	format = "%+.*g";
 	break;
       default:
-	format = "%.7e";
+	format = "%.*g";
       }
       prev_char = '\0';
-      g_string_append_printf(equation, format, val);
+      g_string_append_printf(equation, format, precision, val);
       break;
     case ' ':
       break;
@@ -610,12 +622,26 @@ show_user_equation(struct fitlocal *fitlocal, const char *func, MathValue *par,
       g_string_append_c(equation, toupper(prev_char));
     }
   }
+  return g_string_free(equation, FALSE);
+}
 
-  fitlocal->equation = g_string_free(equation, FALSE);
+static int
+show_user_equation(struct fitlocal *fitlocal, const char *func, MathValue *par, int disp)
+{
+  char *eqn;
   if (disp) {
-    display_equation(fitlocal->equation);
+    eqn = get_user_equation(func, par, disp);
+    if (eqn == NULL) {
+      return 1;
+    }
+    display_equation(eqn);
+    g_free(eqn);
   }
-
+  eqn = get_user_equation(func, par, FALSE);
+  if (eqn == NULL) {
+    return 1;
+  }
+  fitlocal->equation = eqn;
   return 0;
 }
 
