@@ -400,7 +400,6 @@ static int
 draw_gra(struct gra_cache *cache, struct gra_info *info)
 {
   struct GRAdata *data;
-  int rcode;
 
   if (cache == NULL) {
     return 0;
@@ -411,6 +410,7 @@ draw_gra(struct gra_cache *cache, struct gra_info *info)
     return 0;
   }
   while (data) {
+    int rcode;
     rcode = draw_gra_data(info, data);
     if (! rcode) {
       return 1;
@@ -567,7 +567,6 @@ mergedraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   }
   fclose(fd);
 #endif  /* USE_MERGE_CACHE */
-  GRAaddlist(info.GC,obj,inst,(char *)argv[0],"redraw");
   return 0;
 }
 
@@ -614,7 +613,6 @@ mergeredraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv
       GRAlinestyle(GC, 0, NULL, 10, GRA_LINE_CAP_BUTT, GRA_LINE_JOIN_MITER, 1000);
       GRArectangle(GC, x1, y1, x2, y2, 0);
     }
-    GRAaddlist(GC,obj,inst,(char *)argv[0],(char *)argv[1]);
   }
 #endif  /* USE_MERGE_CACHE */
   return 0;
@@ -635,13 +633,17 @@ mergefile(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
     return 0;
   }
 
-  file = get_utf8_filename(argv[2]);
-  g_free(argv[2]);
-  if (file == NULL) {
-    argv[2] = NULL;
+  if (clear_bbox(obj, inst)) {
+    /* argv[2] cannot be freed when a field returns TRUE. */
     return 1;
   }
 
+  file = get_utf8_filename(argv[2]);
+  if (file == NULL) {
+    return 1;
+  }
+
+  g_free(argv[2]);
   if (ignorepath) {
     file2 = getbasename(file);
     g_free(file);
@@ -650,10 +652,6 @@ mergefile(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
     argv[2] = file;
   }
 
-  if (clear_bbox(obj, inst)) {
-    return 1;
-  }
-
   return 0;
 }
 
@@ -760,11 +758,9 @@ mergeload(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   char *s;
   int len;
-  char *file,*fullname,*oldfile,*mes;
+  char *file,*fullname,*oldfile;
   time_t ftime;
   int mkdata;
-  char buf[2];
-  FILE *fp;
   struct utimbuf tm;
 
   s=(char *)argv[2];
@@ -782,6 +778,7 @@ mergeload(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   }
   if (naccess(file,R_OK)!=0) mkdata=TRUE;
   else {
+    char *mes;
     if ((mes=g_malloc(strlen(file)+256))==NULL) {
       g_free(file);
       return 1;
@@ -791,6 +788,8 @@ mergeload(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
     g_free(mes);
   }
   if (mkdata) {
+    char buf[2];
+    FILE *fp;
     if ((fp=nfopen(file,"wt"))==NULL) {
       error2(obj,ERROPEN,file);
       g_free(file);
@@ -821,35 +820,34 @@ mergestoredum(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **ar
   if (mergelocal->endstore) {
     mergelocal->endstore=FALSE;
     return 1;
-  } else {
-    _getobj(obj,"file",inst,&file);
-    if (file==NULL) return 1;
-    style=3;
-    argv2[0]=(char *)&style;
-    argv2[1]=NULL;
-    if (_exeobj(obj,"date",inst,1,argv2)) return 1;
-    style=0;
-    argv2[0]=(char *)&style;
-    argv2[1]=NULL;
-    if (_exeobj(obj,"time",inst,1,argv2)) return 1;
-    _getobj(obj,"date",inst,&date);
-    if(date == NULL) {
-      date = "1-1-1970";
-    }
-    _getobj(obj,"time",inst,&time);
-    if(time == NULL) {
-      time = "00:00:00";
-    }
-    if ((base=getbasename(file))==NULL) return 1;
-    buf = g_strdup_printf("merge::load_dummy '%s' '%s %s'\n", base, date, time);
-    g_free(base);
-    if (buf == NULL) {
-      return 1;
-    }
-    rval->str=buf;
-    mergelocal->endstore=TRUE;
-    return 0;
   }
+  _getobj(obj,"file",inst,&file);
+  if (file==NULL) return 1;
+  style=3;
+  argv2[0]=(char *)&style;
+  argv2[1]=NULL;
+  if (_exeobj(obj,"date",inst,1,argv2)) return 1;
+  style=0;
+  argv2[0]=(char *)&style;
+  argv2[1]=NULL;
+  if (_exeobj(obj,"time",inst,1,argv2)) return 1;
+  _getobj(obj,"date",inst,&date);
+  if(date == NULL) {
+    date = "1-1-1970";
+  }
+  _getobj(obj,"time",inst,&time);
+  if(time == NULL) {
+    time = "00:00:00";
+  }
+  if ((base=getbasename(file))==NULL) return 1;
+  buf = g_strdup_printf("merge::load_dummy '%s' '%s %s'\n", base, date, time);
+  g_free(base);
+  if (buf == NULL) {
+    return 1;
+  }
+  rval->str=buf;
+  mergelocal->endstore=TRUE;
+  return 0;
 }
 
 static int
