@@ -117,7 +117,7 @@ static int
 sysinit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   char *wd;
-  int expand, pid;
+  int expand, pid, status;
   char *exdir;
 
   if (_exeparent(obj,(char *)argv[1],inst,rval,argc,argv)) return 1;
@@ -137,6 +137,8 @@ sysinit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   if ((wd=ngetcwd())==NULL) return 1;
   pid = getpid();
   if (_putobj(obj,"pid",inst,&pid)) return 1;
+  status = 0;
+  if (_putobj(obj, "shell_status", inst, &status)) return 1;
   if (_putobj(obj,"cwd",inst,wd)) {
     g_free(wd);
     return 1;
@@ -174,10 +176,10 @@ static int
 sysdone(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   struct objlist *objcur;
-  int i, n;
+  int i, n, status;
   char *s;
   struct narray *array;
-  struct objlist *objectcur,*objectdel;
+  struct objlist *objectcur;
 
   if (_exeparent(obj,(char *)argv[1],inst,rval,argc,argv)) return 1;
 
@@ -192,6 +194,9 @@ sysdone(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
     }
     objcur=objcur->next;
   }
+  status = 0;
+  _getobj(obj, "shell_status", inst, &status);
+
   _getobj(obj,"conf_dir",inst,&s);
   g_free(s);
   _getobj(obj,"data_dir",inst,&s);
@@ -230,6 +235,7 @@ sysdone(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 
   objectcur=chkobjroot();
   while (objectcur!=NULL) {
+    struct objlist *objectdel;
     objectdel=objectcur;
     objectcur=objectcur->next;
 
@@ -246,7 +252,7 @@ sysdone(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 
   nhash_free(Plugins);
 
-  exit(0);
+  exit(status);
   return 0;
 }
 
@@ -338,7 +344,7 @@ sysunlink(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   char *tmpfil;
   struct narray *array;
-  int i,num;
+  int num;
   char **data;
 
   _getobj(obj,"temp_list",inst,&array);
@@ -346,6 +352,7 @@ sysunlink(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   num=arraynum(array);
   data=arraydata(array);
   if (num>0) {
+    int i;
     tmpfil=(char *)argv[2];
     if (tmpfil==NULL) tmpfil=data[num-1];
     for (i=num-1;i>=0;i--)
@@ -369,18 +376,19 @@ static int
 syshideinstance(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   struct narray *array;
-  struct objlist *obj2;
   char **objdata;
   int j,anum;
 
   array=(struct narray *)argv[2];
-  if (array!=NULL) {
-    anum=arraynum(array);
-    objdata=arraydata(array);
-    for (j=0;j<anum;j++) {
-      obj2=getobject(objdata[j]);
-      if ((obj2!=obj) && (obj2!=NULL)) hideinstance(obj2);
-    }
+  if (array == NULL) {
+    return 0;
+  }
+  anum=arraynum(array);
+  objdata=arraydata(array);
+  for (j=0;j<anum;j++) {
+    struct objlist *obj2;
+    obj2=getobject(objdata[j]);
+    if ((obj2!=obj) && (obj2!=NULL)) hideinstance(obj2);
   }
   return 0;
 }
@@ -389,18 +397,19 @@ static int
 sysrecoverinstance(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   struct narray *array;
-  struct objlist *obj2;
   char **objdata;
   int j,anum;
 
   array=(struct narray *)argv[2];
-  if (array!=NULL) {
-    anum=arraynum(array);
-    objdata=arraydata(array);
-    for (j=0;j<anum;j++) {
-      obj2=getobject(objdata[j]);
-      recoverinstance(obj2);
-    }
+  if (array==NULL) {
+    return 0;
+  }
+  anum=arraynum(array);
+  objdata=arraydata(array);
+  for (j=0;j<anum;j++) {
+    struct objlist *obj2;
+    obj2=getobject(objdata[j]);
+    recoverinstance(obj2);
   }
   return 0;
 }
@@ -414,10 +423,11 @@ systemresize(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **arg
 
   iarray=(struct narray *)argv[2];
   num=arraynum(iarray);
-  if (num >= 2) {
-    data=arraydata(iarray);
-    resizeconsole(data[0], data[1]);
+  if (num < 2) {
+    return 0;
   }
+  data=arraydata(iarray);
+  resizeconsole(data[0], data[1]);
   return 0;
 }
 
@@ -470,7 +480,7 @@ get_plugin_name(const char *name)
 }
 
 static int
-load_plugin_sub(struct objlist *obj, N_VALUE *inst, const char *name, struct ngraph_plugin *plugin)
+load_plugin_sub(struct objlist *obj, const char *name, struct ngraph_plugin *plugin)
 {
   GModule *module;
   void *loaded;
@@ -542,11 +552,10 @@ get_plugin_from_name(const char *name)
 }
 
 struct ngraph_plugin *
-load_plugin(struct objlist *obj, N_VALUE *inst, const char *arg, int *rval)
+load_plugin(struct objlist *obj, const char *arg, int *rval)
 {
   char *name;
   struct ngraph_plugin *plugin;
-  int r;
 
   if (arg == NULL) {
     error(obj, ERRSYSNOMODULE);
@@ -573,7 +582,7 @@ load_plugin(struct objlist *obj, N_VALUE *inst, const char *arg, int *rval)
     return NULL;
   }
 
-  if (load_plugin_sub(obj, inst, name, plugin)) {
+  if (load_plugin_sub(obj, name, plugin)) {
     g_free(name);
     g_free(plugin);
     return NULL;
@@ -581,6 +590,7 @@ load_plugin(struct objlist *obj, N_VALUE *inst, const char *arg, int *rval)
   g_free(name);
 
   if (plugin->open) {
+    int r;
     r = plugin->open();
     if (rval) {
       *rval = r;
@@ -600,7 +610,7 @@ system_plugin_load(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
   struct ngraph_plugin *plugin;
 
   rval->i = 0;
-  plugin = load_plugin(obj, inst, argv[2], &rval->i);
+  plugin = load_plugin(obj, argv[2], &rval->i);
 
   return (plugin) ? 0 : 1;
 }
@@ -692,7 +702,7 @@ system_plugin_exec(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
 
   plugin = get_plugin_from_name(argv[2]);
   if (plugin == NULL) {
-    plugin = load_plugin(obj, inst, argv[2], NULL);
+    plugin = load_plugin(obj, argv[2], NULL);
     if (plugin == NULL) {
       return 1;
     }
@@ -789,7 +799,6 @@ int
 system_set_exec_func(const char *name, ngraph_plugin_exec func)
 {
   struct ngraph_plugin *plugin;
-  int r;
 
   if (name == NULL || func == NULL) {
     return 1;
@@ -797,6 +806,7 @@ system_set_exec_func(const char *name, ngraph_plugin_exec func)
 
   plugin = get_plugin_from_name(name);
   if (plugin == NULL) {
+    int r;
     plugin = g_malloc0(sizeof(struct ngraph_plugin));
     if (plugin == NULL) {
       return 1;
@@ -834,6 +844,7 @@ static struct objtable nsystem[] = {
   {"plugin_dir",NSTR,NREAD,NULL,NULL,0},
   {"home_dir",NSTR,NREAD,NULL,NULL,0},
   {"pid",NINT,NREAD,NULL,NULL,0},
+  {"shell_status",NINT,NREAD,NULL,NULL,0},
   {"time",NSFUNC,NREAD|NEXEC,systime,"i",0},
   {"date",NSFUNC,NREAD|NEXEC,sysdate,"i",0},
   {"temp_file",NSFUNC,NREAD|NEXEC,systemp,"",0},
@@ -861,4 +872,3 @@ addsystem()
   }
   return addobject(NAME,NULL,PARENT,VERSION,TBLNUM,nsystem,ERRNUM,syserrorlist,NULL,NULL);
 }
-
